#include <stdio.h>

#include "sys_ptypes.h"
#include "events.h"
#include "icc.h"
#include "logger.h"
#include "status.h"
#include "pcdapi.h"


#include "bpi_interface.h"
#include "channel_manager.h"
#include "cmd_mbox.h"
#include "context_owners.h"
#include "docint.h"
#include "docsis_logger.h"
#include "docsis_pcd.h"
#include "docsis_db.h"
#include "downstream_manager_api.h"
#include "event_mbox.h"
#include "hal_bpi.h"
#include "hal_db.h"
#include "hal_ds_channel_control.h"
#include "hal_interrupt_cause.h"
#include "hal_mdd.h"
#include "hal_phy_regs.h"
#include "hal_reg_access.h"
#include "hal_sync.h"
#include "mdd_handler.h"
#include "mlx.h"
#include "sys_cfg_db.h"
#include "us_handler_api.h"
#include "us_power_control_manager.h"
#include "hal_phy_regs.h"
#include "hal_phy_top.h"


#define MAP_LOSS_US0_ISR_NUM  34
#define MDD_ARRIVAL_DS0_ISR_NUM 200
#define MDD_ARRIVAL_DS8_ISR_NUM 360
#define MDD_LOSS_DS0_ISR_NUM 64
#define PHY_INT_GROUP_FIRST_ISR         224
#define PHY_INT_GROUP_LAST_ISR          255
#define REMOTE_PHY_INT_GROUP_FIRST_ISR  288
#define REMOTE_PHY_INT_GROUP_LAST_ISR   319
#define FEC_LOST_DS0_ISR_NUM            PHY_INT_GROUP_FIRST_ISR
#define FEC_LOCK_DS0_ISR_NUM            (FEC_LOST_DS0_ISR_NUM + 1)
#define FEC_LOST_DS4_ISR_NUM            REMOTE_PHY_INT_GROUP_FIRST_ISR
#define FEC_LOCK_DS4_ISR_NUM            (FEC_LOST_DS4_ISR_NUM + 1)


#define MAP_IS_ACTIVE_US0_ISR_NUM  128
#define PHY_ERR_US4_ISR_NUM 48
#define RNG_REQ_SENT_US0_ISR_NUM 163
#define RNG_BRC_OPPERT_REC_US0 167

/**************************************************************************/
/*      LOCAL DECLARATIONS:                                               */
/**************************************************************************/
static STATUS HAL_IsrSyncLossHandler(Uint32 isrNumber);
static STATUS HAL_IsrPhyErrHandler(Uint32 isrNumber);
static STATUS HAL_IsrRngReqSent(Uint32 isrNumber);
static STATUS HAL_IsrRxFecLockHandler(Uint32 isrNumber);
static STATUS HAL_IsrRxFecLostHandler(Uint32 isrNumber);
static STATUS HAL_IsrBpiKeySeqErrHandler(Uint32 isrNumber);
static STATUS HAL_IsrMddArrivalHandler(Uint32 isrNumber);
static STATUS HAL_IsrPgaGrtHandler();
static STATUS HAL_IsrMapLossHandlerUs(Uint32 isrNumber);  /* Map has not arrived for over 1 second for upstream 1 */
static STATUS HAL_IsrUcdChangeMapActive(Uint32 isrNumber);
static STATUS HAL_IsrBcstRngOpportunity(Uint32 isrNumber);
static STATUS HAL_IsrMptIntHandler();

/* interrupt cause table */
DST_t mlxtbl[MAX_CAUSE] = {
    /* MAC_ERR1_INT_GROUP */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 0 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 1 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 2 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 3 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 4 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 5 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 6 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 7 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 8 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 9 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 10 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 11 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 12 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 13 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 14 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 15 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 16 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 17 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 18 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 19 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 20 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 21 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 22 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 23 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 24 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 25 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 26 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 27 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 28 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 29 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 30 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 31 */
    /* MAC_ERR2_INT_GROUP */                                        
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 0  */                        
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrSyncLossHandler },          /* 1  */   
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrMapLossHandlerUs },         /* 2  */ /* MAP lost for CH04 */
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrMapLossHandlerUs },         /* 3  */ /* MAP lost for CH03 */
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrMapLossHandlerUs },         /* 4  */ /* MAP lost for CH02 */
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrMapLossHandlerUs },         /* 5  */ /* MAP lost for CH01 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 6  */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 7  */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 8  */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 9  */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 10 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 11 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 12 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 13 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 14 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 15 */ 
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrPhyErrHandler},             /* 16 */ /* PHY error for US04 */ 
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrPhyErrHandler},             /* 17 */ /* PHY error for US03 */
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrPhyErrHandler},             /* 18 */ /* PHY error for US02 */
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrPhyErrHandler},             /* 19 */ /* PHY error for US01 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 20 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 21 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 22 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 23 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 24 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 25 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 26 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 27 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 28 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 29 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 30 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 31 */ 
    /* MAC_ERR3_INT_GROUP */                                        
    { MLX_NO_DEST_CONTEXT, NULL /*Rx1MddLoss*/  },                  /* 0  */ 
    { MLX_NO_DEST_CONTEXT, NULL /*Rx2MddLoss*/  },                  /* 1  */ 
    { MLX_NO_DEST_CONTEXT, NULL /*Rx3MddLoss*/  },                  /* 2  */ 
    { MLX_NO_DEST_CONTEXT, NULL /*Rx4MddLoss*/  },                  /* 3  */ 
    { MLX_NO_DEST_CONTEXT, NULL /*Rx5MddLoss*/  },                  /* 4  */ 
    { MLX_NO_DEST_CONTEXT, NULL /*Rx6MddLoss*/  },                  /* 5  */ 
    { MLX_NO_DEST_CONTEXT, NULL /*Rx7MddLoss*/  },                  /* 6  */ 
    { MLX_NO_DEST_CONTEXT, NULL /*Rx8MddLoss*/  },                  /* 7  */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 8  */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 9  */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 10 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 11 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 12 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 13 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 14 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 15 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 16 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 17 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 18 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 19 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 20 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 21 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 22 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 23 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 24 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 25 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 26 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 27 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 28 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 29 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 30 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 31 */ 
    /* MAC_ERR4_INT_GROUP */                                        
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 0 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 1 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 2 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 3 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 4 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 5 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 6 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 7 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 8 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 9 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 10*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 11*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 12*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 13*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 14*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 15*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 16*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 17*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 18*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 19*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 20*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 21*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 22*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 23*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 24*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 25*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 26*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 27*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 28*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 29*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 30*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 31*/ 
    /* MAC_UCD_INT_GROUP */                                         
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrUcdChangeMapActive },       /* 0  */ 
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrUcdChangeMapActive },       /* 1  */ 
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrUcdChangeMapActive },       /* 2  */ 
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrUcdChangeMapActive },       /* 3  */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 4  */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 5  */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 6  */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 7  */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 8  */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 9  */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 10 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 11 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 12 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 13 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 14 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 15 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 16 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 17 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 18 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 19 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 20 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 21 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 22 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 23 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 24 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 25 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 26 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 27 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 28 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 29 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 30 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 31 */
    /* MAC_US_FW_INT_GROUP */                                        
    { MLX_NO_DEST_CONTEXT, (func)HAL_CmdMbox_SendCmdCmplt },        /* 0  */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 1  */
    { MLX_NO_DEST_CONTEXT, (func)HAL_EventMbox_SendEventReceived }, /* 2  */
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrRngReqSent },               /* 3  */
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrRngReqSent },               /* 4  */
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrRngReqSent },               /* 5  */
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrRngReqSent },               /* 6  */
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrBcstRngOpportunity },
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrBcstRngOpportunity },
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrBcstRngOpportunity },
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrBcstRngOpportunity },
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 11 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 12 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 13 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 14 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 15 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 16 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 17 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 18 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 19 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 20 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 21 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 22 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 23 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 24 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 25 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 26 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 27 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 28 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 29 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 30 */
    { MLX_NO_DEST_CONTEXT, (func)HAL_CmdMbox_SendMboxReady },       /* 31 */
    /* MAC_DS_FW_INT_GROUP */                                       
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrBpiKeySeqErrHandler },      /* 0  */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 1  */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 2  */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 3  */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 4  */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 5  */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 6  */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 7  */
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrMddArrivalHandler },        /* 8  */
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrMddArrivalHandler },        /* 9  */
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrMddArrivalHandler },        /* 10 */
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrMddArrivalHandler },        /* 11 */
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrMddArrivalHandler },        /* 12 */
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrMddArrivalHandler },        /* 13 */
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrMddArrivalHandler },        /* 14 */
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrMddArrivalHandler },        /* 15 */
    { MLX_NO_DEST_CONTEXT, (func)HAL_MacDsSeqOutOfRangeIsrHandler },/* 16 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 17 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 18 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 19 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 20 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 21 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 22 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 23 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 24 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 25 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 26 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 27 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 28 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 29 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 30 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 31 */
    /* PHY_INT_GROUP */                                             
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrRxFecLostHandler },        /* 0 */  /*Rx1: 0-5*/
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrRxFecLockHandler },        /* 1 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 2 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 3 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 4 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 5 */
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrRxFecLostHandler },         /* 6 */ /*Rx2: 6-11*/
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrRxFecLockHandler },        /* 7 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 8 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 9 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 10*/
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 11*/
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrRxFecLostHandler },         /* 12*/ /*Rx3: 12-17*/
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrRxFecLockHandler },         /* 13*/
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 14*/
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 15*/
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 16*/
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 17*/
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrRxFecLostHandler },         /* 18*/     /*Rx4: 18-23*/
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrRxFecLockHandler },        /* 19*/
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 20*/
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 21*/
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 22*/
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 23*/
    { MLX_NO_DEST_CONTEXT, NULL },/*24-28*/                         /* 24*/
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 25*/
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 26*/
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 27*/
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 28*/
    { MLX_NO_DEST_CONTEXT, NULL },/*spare*/                         /* 29*/
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 30*/
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 31*/
    /* PGA GRT INTERRUPT */                                         
    { MLX_NO_DEST_CONTEXT, HAL_IsrPgaGrtHandler }, /*TBD*/          /* 0*/
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 1 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 2 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 3 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 4 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 5 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 6 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 7 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 8 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 9 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 10*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 11*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 12*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 13*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 14*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 15*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 16*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 17*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 18*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 19*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 20*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 21*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 22*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 23*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 24*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 25*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 26*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 27*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 28*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 29*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 30*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 31*/ 
    /* REMOTE_PHY_INT_GROUP */                                             
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrRxFecLostHandler },         /* 0 */  /*Rx4: 0-5*/
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrRxFecLockHandler },         /* 1 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 2 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 3 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 4 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 5 */
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrRxFecLostHandler },         /* 6 */  /*Rx5: 6-11*/
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrRxFecLockHandler },         /* 7 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 8 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 9 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 10*/
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 11*/
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrRxFecLostHandler },         /* 12*/  /*Rx6: 12-17*/
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrRxFecLockHandler },         /* 13*/
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 14*/
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 15*/
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 16*/
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 17*/
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrRxFecLostHandler },         /* 18*/   /*Rx7: 18-23*/
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrRxFecLockHandler },         /* 19*/
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 20*/
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 21*/
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 22*/
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 23*/
    { MLX_NO_DEST_CONTEXT, NULL },/*24-28*/                         /* 24*/
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 25*/
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 26*/
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 27*/
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 28*/
    { MLX_NO_DEST_CONTEXT, NULL },/*spare*/                         /* 29*/
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 30*/
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 31*/
    /* MPT_INT */                                         
    { MLX_NO_DEST_CONTEXT, HAL_IsrMptIntHandler},                   /* 0*/
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 1 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 2 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 3 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 4 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 5 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 6 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 7 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 8 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 9 */ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 10*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 11*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 12*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 13*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 14*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 15*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 16*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 17*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 18*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 19*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 20*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 21*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 22*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 23*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 24*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 25*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 26*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 27*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 28*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 29*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 30*/ 
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 31*/ 
    /* REMOTE_MAC_INT */                                         
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrBpiKeySeqErrHandler },      /* 0  */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 1  */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 2  */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 3  */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 4  */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 5  */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 6  */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 7  */
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrMddArrivalHandler },        /* 8  */
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrMddArrivalHandler },        /* 9  */
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrMddArrivalHandler },        /* 10 */
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrMddArrivalHandler },        /* 11 */
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrMddArrivalHandler },        /* 12 */
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrMddArrivalHandler },        /* 13 */
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrMddArrivalHandler },        /* 14 */
    { MLX_NO_DEST_CONTEXT, (func)HAL_IsrMddArrivalHandler },        /* 15 */
    { MLX_NO_DEST_CONTEXT, (func)HAL_RemoteMacDsSeqOutOfRangeIsrHandler },/* 16 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 17 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 18 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 19 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 20 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 21 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 22 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 23 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 24 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 25 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 26 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 27 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 28 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 29 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 30 */
    { MLX_NO_DEST_CONTEXT, NULL },                                  /* 31 */

};                                                                  
/**************************************************************************/
/*      LOCAL FUNCTIONS:                                                  */
/**************************************************************************/
static STATUS HAL_IsrSyncLossHandler(Uint32 isrNumber)
{
    Uint32 primaryIdx;

    HAL_SyncTimeoutNotificationEnable(False);

    /* Get the index of the primary DS */
    HalDb_GetPrimaryDsReceiver(&primaryIdx);

    TI_LOG_INFO(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, 
                "DS[%d]: SYNC Lost - Notidy to DSM.", primaryIdx+1);

    /* invoke the DS Manager's handler function */
    DSM_NotifySyncLost(primaryIdx+1);

    return STATUS_OK;
}

static STATUS HAL_IsrPhyErrHandler(Uint32 isrNumber)
{
    /* portId starts from 0-3 - interrupt lines are going from port 3 to 0 */
    Uint8 port = 3 - (isrNumber - PHY_ERR_US4_ISR_NUM);
    Uint16 reg85 = PHY_TX_CH_REG_OFFSET(port, PHY_STATUS_1_REGISTER);
    Uint16 reg86 = PHY_TX_CH_REG_OFFSET(port, PHY_STATUS_2_SCDMA_REGISTER);
    Uint16 value85 = 0;
    Uint16 value86 = 0;
    
    /* Need a dummy-write to register 0x85 to lock 0x85, 0x86 */
    /* Read From Register 0x85 clears registers 0x85, 0x86 (so first read 0x86) */
    HAL_SetPhyReg16(reg85, value85);

    HAL_GetPhyReg16(reg86, &value86);
    HAL_GetPhyReg16(reg85, &value85);

    TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, " port%d PHY ERROR. SCDMA Status1 register = 0x%x, SCDMA Status2 register = 0x%x", port, value85, value86);

    if (value85 & 0x1)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "error_frame_in1 - Framer error. Framer gets f_start while framer in the middle of filling the frame.");
    }
    if (value85 & 0x2)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "error_frame_in2 - Framer error. Framer gets frame_start on a frame that transmitted.");
    }
    if (value85 & 0x4)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "error_frame_out - Framer error. Framer gets f_start before finish transmitting prev. frame.");
    }
    if (value85 & 0x8)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "error_framer_size - Framer error. Framer gets more symbols then he suppose to get.");
    }
    if (value85 & 0x10)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "error_frame_skip - Framer error. Framer gets f_start with frame num, which passed the frame number for transmission.");
    }
    if (value85 & 0x20)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "error_req_cnt - Spreader error, request count error.");
    }
    if (value85 & 0x40)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "error_ready - Spreader error. Ready error.");
    }
    if (value85 & 0x80)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "error_switch - Spreader error. Switch error.");
    }
    if (value85 & 0x100)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "error_switch_in_ff - Spreader error. Switch in middle of fast forward error.");
    }
    if (value85 & 0x200)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "error_b_param - Tx decision error. The burst parameters for the next frame are too far ahead/behind (more than 100 ms).");
    }
    if (value85 & 0x400)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "error_f_passed - Tx decision error. The incoming burst arrived too late - the frame has been transmitted.");
    }
    if (value85 & 0x800)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "error_b_start - Tx decision error. The burst parameters did not enter the framer and a new burst arrived.");
    }
    if (value85 & 0x1000)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "error_start_in_ucd  - Tx decision error. Error indication that during a wait for UCD change a burst start from the MAC arrived.");
    }
    if (value85 & 0x2000)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "Error Rs_cnt - Rs error RS get more bytes than he supposed to get.");
    }
    if (value85 & 0x4000)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "Error burst start - Burst transmission time is over - STDMA only.");
    }
    if (value85 & 0x8000)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "Error state-machine - Start FIFO state machine is stuck - STDMA only");
    }
    

    if (value86 & 0x1)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "Overflow EQL - Pre-equalizer overflow.");
    }
    if (value86 & 0x2)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "Overflow PSF - Shaping filter overflow.");
    }
    if (value86 & 0x4)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "Overflow HBF - HBF overflow.");
    }
    if (value86 & 0x8)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "Overflow Upconv - Cubic overflow.");
    }
    if (value86 & 0x10)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "Overflow Atten.");
    }
    if (value86 & 0x20)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "Error STDMA ready.");
    }
    if (value86 & 0x40)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "error_a_start_fifo_s - Start FIFO - start within start before it reached BI (ATDMA).");
    }
    if (value86 & 0x80)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "error_a_start_fifo_of_s - Start FIFO overflow (ATDMA).");
    }
    if (value86 & 0x100)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "error_a_bi_miss_start_s - BI missed a start signal (ATDMA)");
    }
    if (value86 & 0x200)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "error_a_bi_start_s - Byte interleave received start before a block is ready (ATDMA).");
    }
    if (value86 & 0x400)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "bi_ram_empty - BI RAM empty (ATDMA).");
    }
    if (value86 & 0x800)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "amp_en - Amplifier enable (high when transmitting).");
    }
    
    TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "burst_profile_fifo = %d - Upstream current working profile.", ((value86 >> 12) & 0xF));

    return STATUS_OK;
}



static STATUS HAL_IsrRxFecLockHandler(Uint32 isrNumber)
{
    Uint32 portId;
    Bool secondaryPumaExist;
    ANNEX_e phyAnnex = PHY_ANNEX_B;
    Uint16 pValue;

    SysCfgDb_GetBoardConfigIsSecondaryPumaSupported(&secondaryPumaExist);
    
    if (isrNumber >= PHY_INT_GROUP_FIRST_ISR && isrNumber <= PHY_INT_GROUP_LAST_ISR)
    {
        /* portId range is 1-4 */
        portId = (isrNumber - FEC_LOCK_DS0_ISR_NUM)/6 + 1;
    }
    else if (isrNumber >= REMOTE_PHY_INT_GROUP_FIRST_ISR && isrNumber <= REMOTE_PHY_INT_GROUP_LAST_ISR)
    {
        if(secondaryPumaExist)
        {
            /* portId range is 9-12 */
            portId = (isrNumber - FEC_LOCK_DS4_ISR_NUM)/6 + 9;
        }
        else
        {
            /* portId range is 5-8 */
            portId = (isrNumber - FEC_LOCK_DS4_ISR_NUM)/6 + 5;
        }
    }
    else
    {
        return STATUS_NOK;
    }

    /* This section is relevant only for Euro case. */
    PHY_Single_DS_Get_Annex_Type((PHY_RECEIVER_CHANNEL_e)(portId-1),&phyAnnex);
    if (PHY_ANNEX_A == phyAnnex /* Euro */)
    {
        /* If one of QAM, MPEG or FEC are not lock then this is a false QamFecRecovery interrupt */
        if (PHY_IsQamLocked((PHY_RECEIVER_CHANNEL_e)(portId-1)) == False ||
            PHY_IsMpegLocked((PHY_RECEIVER_CHANNEL_e)(portId-1)) == False ||
            PHY_IsFecLocked((PHY_RECEIVER_CHANNEL_e)(portId-1), phyAnnex) == False)
        {
            PHY_RestartSection( PHY_SECTION_2_BIT(PHY_DS_RX_1_RESTART) << (portId -1) );
            TI_LOG_DEBUG(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "Ignore FecRecovery interrupt on DS port %d",portId);
            return STATUS_OK; /* False QamFecRecovery interrupt */
        }

        HAL_SetPhyReg16(PHY_RX_CH_REG_OFFSET( (portId -1) ,PHY_QAM_EXTENDED_SPACE_ADDR_REGISTER),0x86);
        pValue = GET_PHY_REG_FIELD(PHY_RX_CH_REG_OFFSET( (portId -1), PHY_QAM_EXTENDED_SPACE_DATA_REGISTER), MASK_16_BIT, POSITION_0);
        if (pValue < 9) /* This 9 (fagc) is an energy threshold for detecting false FecRecovery */
        {
            PHY_RestartSection( PHY_SECTION_2_BIT(PHY_DS_RX_1_RESTART) << (portId -1) );
            TI_LOG_DEBUG(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "Ignore FecRecovery interrupt on DS port %d (fagc=%d)",portId, pValue);
            return STATUS_OK; /* False QamFecRecovery interrupt */
        }
    }
    
    DSM_NotifyQamFecRecovery(portId);
    return STATUS_OK;
}
static STATUS HAL_IsrRxFecLostHandler(Uint32 isrNumber)
{
    Uint32 portId;
    Bool secondaryPumaExist;

    SysCfgDb_GetBoardConfigIsSecondaryPumaSupported(&secondaryPumaExist);

    if (isrNumber >= PHY_INT_GROUP_FIRST_ISR && isrNumber <= PHY_INT_GROUP_LAST_ISR)
    {
        /* portId range is 1-4 */
        portId = (isrNumber - FEC_LOST_DS0_ISR_NUM)/6 + 1;
    }
    else if (isrNumber >= REMOTE_PHY_INT_GROUP_FIRST_ISR && isrNumber <= REMOTE_PHY_INT_GROUP_LAST_ISR)
    {
        if(secondaryPumaExist)
        {
            /* portId range is 9-12 */
            portId = (isrNumber - FEC_LOST_DS4_ISR_NUM)/6 + 9;
        }
        else
        {
            /* portId range is 5-8 */
            portId = (isrNumber - FEC_LOST_DS4_ISR_NUM)/6 + 5;
        }
    }
    else
    {
        return STATUS_NOK;
    }

    DSM_NotifyQamFecLost(portId);
    return STATUS_OK;
}

static STATUS HAL_IsrBpiKeySeqErrHandler(Uint32 isrNumber)
{
    Int16 index = -1;

    /* Get the SAID from HW - TBD */
    if(HAL_BpiDsGetFirstKeySeqErrIndex(&index) != STATUS_OK)
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "Failed to get SAID index from HW");
        return  STATUS_NOK;
    }

    if(index == -1)
    {
        TI_LOG_WARNING(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "Received Key-Seq error interrupt, but no SAID index is set in HW");
        HAL_BpiDsEnableKeySeqErrorNotification(True);
        return  STATUS_NOK;
    }
    /* Envoke the BPI API for notifying TEK-Invalid */
    if(BpiIfc_TekSeqNumInv(index) != STATUS_OK )
    {                          
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "Failed to send Teq-Seq-Invalid message to BPI SM");
        return STATUS_NOK;
    }

    return STATUS_OK;
}

/* Pdsp - MDD Arrival (general) */
static STATUS HAL_IsrMddArrivalHandler(Uint32 isrNumber)
{
    Uint32 portId;
    /* Port ID start from 1 */
    if(isrNumber >= MDD_ARRIVAL_DS8_ISR_NUM)
    {
        portId = ((isrNumber - MDD_ARRIVAL_DS8_ISR_NUM) + 1 + MAX_PUMA_DS_PORTS);
    }
    else
    {
        portId = (isrNumber - MDD_ARRIVAL_DS0_ISR_NUM + 1);
    }
    /* Note: Further notifications of the same event are masked */
    /* by the kernel-level handler. Will be unmasked again next time */
    /* an MDD-Loss event is reported by h/w. */

    /* Notify DS manager -- MDD Arrival */
    if (DSM_NotifyMddArrival(portId) != DSM_RS_OK)
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, 
                    "DS[%d]: MDD Recovered - Failed to notify DS manager!", portId);

    return STATUS_OK;
}

static STATUS HAL_IsrPgaGrtHandler()
{
    UPCM_HandleCancelWaitForGRT();
    return (STATUS_OK);
}

static STATUS HAL_IsrMapLossHandlerUs(Uint32 isrNumber)
{
    Uint32 usPort;

    /* Port ID start from 1 - interrupt lines are going from port 3 to 0*/
    usPort = 4 - (isrNumber -  MAP_LOSS_US0_ISR_NUM);
    TI_LOG_INFO(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, 
                "US[%d]: MAPs lost", usPort);
    EV_GenEvent(ICC_dest_of(CTX_CHM), CHM_ev_MapsLost, &usPort, sizeof(usPort));

    return STATUS_OK;
}

static STATUS HAL_IsrRngReqSent(Uint32 isrNumber)
{
    CTX_ContextOwners_e contextOwner = CTX_USM1;
    uint32 usPort = (isrNumber - RNG_REQ_SENT_US0_ISR_NUM);

    EV_GenEvent(ICC_dest_of(contextOwner + usPort), USM_ev_RngReqSent, NULL, 0);

    return STATUS_OK;
}

static STATUS HAL_IsrUcdChangeMapActive(Uint32 isrNumber)
{
    /* Port ID starts from 1-4 */
    Uint8 portId = (Uint8)(isrNumber - MAP_IS_ACTIVE_US0_ISR_NUM + 1);
    USM_UcdChangeMapActivePort(portId);

    return STATUS_OK;
}

static STATUS HAL_IsrBcstRngOpportunity(Uint32 isrNumber)
{
    const Uint32 ctxOfPort[] = {CTX_USM1, CTX_USM2, CTX_USM3, CTX_USM4};
    /* Port Index starts from 0-3 */
    Uint8 portIndex = (Uint8)(isrNumber - RNG_BRC_OPPERT_REC_US0);

    TI_LOG_DEBUG(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, 
                "Got ISR from US FW it recieve BRC IUC3 for portIndex = %d", portIndex);
	
    EV_GenEvent(ICC_dest_of(ctxOfPort[portIndex]), USM_ev_BcstRngOpportunity, NULL, 0);
    
    return STATUS_OK;
}


static STATUS HAL_IsrMptIntHandler()
{
    ICC_message_t *iccMsg;

    if(!(iccMsg = ICC_alloc_message(-1, 0, True)))
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, "ICC_alloc_message failed");
         return (STATUS_NOK);
    }

    if (ICC_send_message((ICC_context)ICC_dest_of(CTX_MPT), iccMsg, 0) == -1) 
    {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0," ICC_send_message failed");
        ICC_dispose_message(iccMsg);
        return (STATUS_NOK);
    }
    
    return (STATUS_OK);
}
