/*
 *
 * hal_docsis_cppi.h
 * Description:
 * Declaration of types of the DOCSIS CPPI.
 *
 * Copyright (C) 2008 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifndef __DOCSIS_CPPI_REGS_H__
#define __DOCSIS_CPPI_REGS_H__

#include "_tistdtypes.h"
#include "puma5_hardware.h"


static inline void bit_field32_set(Uint32 *addr, Uint32 data, Uint32 offset, Uint32 width)
{
    Uint32 mask;

    mask = ((0xFFFFFFFF >> (32-width)) << offset);
    
    *addr &= ~mask;
    *addr |=  mask & (data << offset);
}

/************************************************************************/
/*                                                                      */
/*     General Definitions                                              */
/*                                                                      */
/************************************************************************/
#define HAL_MAC_BASE_ADDRESS_PHYS             0x9000000
#define HAL_MAC_BASE_ADDRESS                  IO_PHY2VIRT(HAL_MAC_BASE_ADDRESS_PHYS)


/************************************************************************/
/*                                                                      */
/*      DOCSIS Buffer Manager specific definition                       */
/*                                                                      */
/************************************************************************/
#define DOCSIS_MAC_DS_INTERNAL_RAM_BASE_ADDR  0x01040000

#define DS_FREE_BUFFER_POOL_INDEX             0
#define DS_FREE_BUFFER_POOL_SIZE              1     /* 64 buffers */
#define DS_FREE_BUFFER_POOL_BUFFS_SIZE        3     /* 512 bytes  */

/************************************************************************/
/*                                                                      */
/*      DOCSIS Buffer Manager (BMGR) Control Registers                  */
/*                                                                      */
/************************************************************************/
#define DOCSIS_BUFFER_MNG_BASE_ADDR     0x00068000
#define CPPI_BMGR_POOL_ENTRY_SIZE       0x8

#define DOCSIS_BMGR_BUF_REF_CNT_INC_VAL                 (DOCSIS_BUFFER_MNG_BASE_ADDR + 0x0028)
#define DOCSIS_BMGR_BUF_REF_CNT_INC_PTR                 (DOCSIS_BUFFER_MNG_BASE_ADDR + 0x002C)
#define DOCSIS_BMGR_FREE_BUF_POOL_N_PTR(pool)           (DOCSIS_BUFFER_MNG_BASE_ADDR + 0x100 + (CPPI_BMGR_POOL_ENTRY_SIZE * (pool)))
#define DOCSIS_BMGR_FREE_BUF_POOL_N_BUF_SIZE(pool)      (DOCSIS_BUFFER_MNG_BASE_ADDR + 0x104 + (CPPI_BMGR_POOL_ENTRY_SIZE * (pool)))
#define DOCSIS_BMGR_FREE_BUF_POOL_N_ADDR(pool)          (DOCSIS_BUFFER_MNG_BASE_ADDR + 0x600 + (CPPI_BMGR_POOL_ENTRY_SIZE * (pool)))
#define DOCSIS_BMGR_FREE_BUF_POOL_N_CTRL(pool)          (DOCSIS_BUFFER_MNG_BASE_ADDR + 0x604 + (CPPI_BMGR_POOL_ENTRY_SIZE * (pool)))
/************************************************************************/
/*      BUFFER MNG FREE POOL CONTROL REG FIELDS ACCESS MACROS           */
/************************************************************************/
#define BMGR_SET_FREE_POOL_SIZE(w0,val)                 bit_field32_set(&(w0),(val),0,4)
#define BMGR_SET_FREE_POOL_BUFF_SIZE(w0,val)            bit_field32_set(&(w0),(val),16,4)
#define BMGR_SET_FREE_POOL_REF_CNT(w0,val)              bit_field32_set(&(w0),(val),24,1)
#define BMGR_SET_FREE_POOL_ENABLE(w0,val)               bit_field32_set(&(w0),(val),31,1)


/************************************************************************/
/*                                                                      */
/*      DOCSIS Queue Manager (QMGR) Control Registers                   */
/*                                                                      */
/************************************************************************/
#define DOCSIS_QUEUE_MNG_BASE_ADDR 0x00060000

/************************************************************************/
/*         Queue Manager Region           (see chapter 6.1)             */
/************************************************************************/
#define DOCSIS_QMGR_REGS_OFFSET     0x00062000
/* Revision Register */
#define DOCSIS_QMGR_REV_REG         ( 0    + DOCSIS_QMGR_REGS_OFFSET  )
#define DOCSIS_QMGR_RESET_REG       ( 0x4  + DOCSIS_QMGR_REGS_OFFSET  )
/* Queue Diversion Register */
#define DOCSIS_QMGR_DIVS_REG        ( 0x8  + DOCSIS_QMGR_REGS_OFFSET  )
/* Free Descriptor Starvation Count Register 0 -6 */
#define DOCSIS_QMGR_STARV_CNT0_REG  ( 0x20 + DOCSIS_QMGR_REGS_OFFSET  )
#define DOCSIS_QMGR_STARV_CNT1_REG  ( 0x24 + DOCSIS_QMGR_REGS_OFFSET  )
#define DOCSIS_QMGR_STARV_CNT2_REG  ( 0x28 + DOCSIS_QMGR_REGS_OFFSET  )
#define DOCSIS_QMGR_STARV_CNT3_REG  ( 0x2C + DOCSIS_QMGR_REGS_OFFSET  )
#define DOCSIS_QMGR_STARV_CNT4_REG  ( 0x30 + DOCSIS_QMGR_REGS_OFFSET  )
#define DOCSIS_QMGR_STARV_CNT5_REG  ( 0x34 + DOCSIS_QMGR_REGS_OFFSET  )
#define DOCSIS_QMGR_STARV_CNT6_REG  ( 0x3C + DOCSIS_QMGR_REGS_OFFSET  )

/* Linking RAM Region 0 Base Address Register */
#define DOCSIS_QMGR_LRAM0_BASE_REG  ( 0x80 + DOCSIS_QMGR_REGS_OFFSET  )
/* Linking RAM Region 0 Size Register */ 
#define DOCSIS_QMGR_LRAM0_SIZE_REG  ( 0x84 + DOCSIS_QMGR_REGS_OFFSET  )
/* Linking RAM Region 1 Base Address Register */
#define DOCSIS_QMGR_LRAM1_BASE_REG  ( 0x88 + DOCSIS_QMGR_REGS_OFFSET  )
/************************************************************************/
/*         Descriptor Memory Setup Region (see chapter 6.2)             */
/************************************************************************/
#define DOCSIS_QMGR_DESC_MEM_SETUP_OFFSET     0x00063000

#define CPPI_QMGR_DESC_ENTRY_SIZE           0x10

#define DOCSIS_QMGR_RAM_BASE_REG(rgn)    ( 0x00 + DOCSIS_QMGR_DESC_MEM_SETUP_OFFSET + (CPPI_QMGR_DESC_ENTRY_SIZE * (rgn)) )
#define DOCSIS_QMGR_RAM_CTRL_REG(rgn)    ( 0x04 + DOCSIS_QMGR_DESC_MEM_SETUP_OFFSET + (CPPI_QMGR_DESC_ENTRY_SIZE * (rgn)) )

#define QMGR_SET_MEM_REGION_DESC_SIZE(w0,val)       bit_field32_set(&(w0),(val),8,4)
#define QMGR_SET_MEM_REGION_START_INDEX(w0,val)     bit_field32_set(&(w0),(val),16,14)
#define QMGR_SET_MEM_REGION_NUM_OF_DESC(w0,val)     bit_field32_set(&(w0),(val),0,4)

/************************************************************************/
/*         Queue Management Region (see chapter 6.3)                    */
/************************************************************************/
#define DOCSIS_QMGR_QUEUE_MANAGMENT_REGION     DOCSIS_QUEUE_MNG_BASE_ADDR


/************************************************************************/
/*                                                                      */
/*      DOCSIS Queue Manager specific definition                        */
/*                                                                      */
/************************************************************************/
#define DS_NUM_OF_EMBEDDE_DESCRIPTORS   32
#define US_NUM_OF_MONO_DESC             32
#define US_NUM_OF_EMBEDDE_DESC          512
#define GENERAL_NUM_OF_DESC             32

#define DOCSIS_LINKING_RAM_0_REGION_SIZE        (DS_NUM_OF_EMBEDDE_DESCRIPTORS + US_NUM_OF_MONO_DESC + US_NUM_OF_EMBEDDE_DESC + GENERAL_NUM_OF_DESC)
#define DOCSIS_LINKING_RAM_0_REGION_BASE_ADDR   0x01080000 /* DOCSIS MAC internal addressing */
#define DOCSIS_LINKING_RAM_1_REGION_BASE_ADDR   0x01081000 /* DOCSIS MAC internal addressing */

/* Downstream channel Embedded Descriptors configuration */
#define DS_EMBEDDED_DESC_REGION_NUM             0

#define DS_EMBEDDED_DESC_RAM_BASE_ADDR_OFFSET       (0x00040000 + 0x8000)
#define DS_EMBEDDED_DESC_RAM_DESC_SIZE              128
#define DS_EMBEDDED_DESC_FREE_QUEUE_REG_OFFSET      0x000603AC

#define  DS_EMBEDDED_DESC_WORD_2_OFFSET 0x8
#define  DS_EMBEDDED_DESC_WORD_1_OFFSET 0x4
#define  DS_EMBEDDED_DESC_WORD_0_OFFSET 0x0

#define  DS_EMBEDDED_DESC_BUFF_0_OFFSET 0xC

/* Set the PS len to 5 */
#define DS_EMBEDDED_DESC_WORD_0_DEFAULT_VAL         0x01400000

#define DS_EMBEDDED_DESC_WORD_1_DEFAULT_VAL         0
/* Return Policy is '1', Desc is in on-chip memory, Packet Return Queue index #1 & Queue #58 (0x3A) */
#define DS_EMBEDDED_DESC_WORD_2_DEFAULT_VAL         0x0000D03A
#define DS_EMBEDDED_DESC_BPI_CMD_OFFSET             0x34
#define DS_EMBEDDED_DESC_CRC_CMD_OFFSET             0x38
#define DS_EMBEDDED_DESC_PHS_CMD_OFFSET             0x3C
#define DS_EMBEDDED_DESC_RX_FIFO_0_CMD_OFFSET       0x40
#define DS_EMBEDDED_DESC_RX_FIFO_1_CMD_OFFSET       0x44

#define DS_EMBEDDED_DESC_BPI_CMD_TEMPLATE           0x21800000
#define DS_EMBEDDED_DESC_CRC_CMD_TEMPLATE           0x41800000
#define DS_EMBEDDED_DESC_PHS_CMD_TEMPLATE           0x11800000
#define DS_EMBEDDED_DESC_RX_FIFO_0_CMD_TEMPLATE     0x72000000
#define DS_EMBEDDED_DESC_RX_FIFO_1_CMD_TEMPLATE     0x080404C1


#define DS_EMBEDDED_DESC_RAM_BASE_ADDR              (0x01040000 + 0x8000)              /* DOCSIS MAC internal addressing */
#define DS_EMBEDDED_DESC_RAM_BASE_ADDR_HOST         IO_PHY2VIRT((0x09040000 + 0x8000)) /* Host addressing */
#define DS_EMBEDDED_DESC_RAM_DESC_ENCODED_SIZE      2  /* descriptor size = 128 bytes */
#define DS_EMBEDDED_DESC_RAM_START_INDEX            0
#define DS_EMBEDDED_DESC_RAM_NUM_OF_DESC            0  /* 0 -> 32 descriptors */

/* Upstream channel Embedded Descriptors configuration */
/* Word 0 - Descriptor contains 4 buffer slot */
#define US_EMBEDDED_DESC_WORD_0_DEFAULT_VAL         0x18000000
#define US_EMBEDDED_DESC_WORD_1_DEFAULT_VAL         0
/* Word 2 - Packet Return Queue #60 - Queue Mgr #1 - Return Policy 1  */
#define US_EMBEDDED_DESC_WORD_2_DEFAULT_VAL         0x903C

#define US_EMBEDDED_DESC_REGION_NUM                         2
/*#define US_EMBEDDED_DESC_DDR_BASE_ADDR                      0x10C00000*/
#define US_EMBEDDED_DESC_RAM_DESC_SIZE                      64

#define US_EMBEDDED_DESC_RAM_DESC_ENCODED_SIZE              1           /* descriptor size = 64 bytes */
#define US_EMBEDDED_DESC_RAM_START_INDEX                    0x40
#define US_EMBEDDED_DESC_RAM_NUM_OF_DESC                    4           /* 0 -> 512 descriptors */
#define US_EMBEDDED_DESC_FREE_QUEUE_REG_OFFSET              0x000603CC

/* Upstream channel Monolithic Descriptors configuration */
#define US_MONOLITHIC_DESC_REGION_NUM               1
#define US_MONOLITHIC_DESC_RAM_BASE_ADDR            0x01119000 /* DOCSIS MAC internal addressing */
#define US_MONOLITHIC_DESC_RAM_DESC_SIZE            128

#define US_MONOLITHIC_DESC_RAM_DESC_ENCODED_SIZE    2   /* descriptor size = 128 bytes */
#define US_MONOLITHIC_DESC_RAM_START_INDEX          0x20
#define US_MONOLITHIC_DESC_RAM_NUM_OF_DESC          0  /* 0 -> 32 descriptors */

/************************************************************************/
/*	DMA Controller - Channel Control / Status Registers Region          */
/************************************************************************/
#define DOCSIS_CPPI_DMA_NUM_RX_CHANNEL  8
#define DOCSIS_CPPI_DMA_NUM_TX_CHANNEL  1
#define DOCSIS_CPPI_DMA_NUM_TX_RX_CHANNEL  (DOCSIS_CPPI_DMA_NUM_RX_CHANNEL + DOCSIS_CPPI_DMA_NUM_TX_CHANNEL)    /* 1 TX channel for Co-processor + 8 RX channels for 8 DS channel */

typedef volatile struct
{
    volatile Uint32 txGlobalConfig;
    volatile Uint32 reserved;
    volatile Uint32 rxGlobalConfig;
    volatile Uint32 hostPktConfigRegA;
    volatile Uint32 hostPktConfigRegB;
    volatile Uint32 embeddedPktConfigRegA;
    volatile Uint32 embeddedPktConfigRegB;
    volatile Uint32 monolithicPktConfigRegA;
} DMAChannelConfigRegs_t;

typedef volatile struct  
{
  volatile DMAChannelConfigRegs_t  channelConfig[DOCSIS_CPPI_DMA_NUM_TX_RX_CHANNEL]; 
} DMAChannelCtrlStatusRegs_t;

/**
 * \brief CPPI4.1 Global control register overlay pointer
 */
typedef DMAChannelCtrlStatusRegs_t *DMAChnCtrlStatusRegsOvly;

/* channel configuration register bits */
#define DMA_CHAN_CTRL_TX_GLOBAL_CHAN_ENABLE_SHIFT           31
#define DMA_CHAN_CTRL_TX_GLOBAL_CHAN_ENABLE_MASK            (1 << DMA_CHAN_CTRL_TX_GLOBAL_CHAN_ENABLE_SHIFT)
#define DMA_CHAN_CTRL_TX_GLOBAL_CHAN_TD_SHIFT               30
#define DMA_CHAN_CTRL_TX_GLOBAL_CHAN_TD_MASK                (1 << DMA_CHAN_CTRL_TX_GLOBAL_CHAN_TD_SHIFT)               
#define DMA_CHAN_CTRL_TX_GLOBAL_DEF_QMGR_SHIFT              12
#define DMA_CHAN_CTRL_TX_GLOBAL_DEF_QMGR_MASK               (0x3 << DMA_CHAN_CTRL_TX_GLOBAL_DEF_QMGR_SHIFT)
#define DMA_CHAN_CTRL_TX_GLOBAL_DEF_QNUM_SHIFT              0
#define DMA_CHAN_CTRL_TX_GLOBAL_DEF_QNUM_MASK               (0xFFF << DMA_CHAN_CTRL_TX_GLOBAL_DEF_QNUM_SHIFT)

#define DMA_CHAN_CTRL_RX_GLOBAL_CHAN_ENABLE_SHIFT           31
#define DMA_CHAN_CTRL_RX_GLOBAL_CHAN_ENABLE_MASK            (1 << DMA_CHAN_CTRL_RX_GLOBAL_CHAN_ENABLE_SHIFT)
#define DMA_CHAN_CTRL_RX_GLOBAL_CHAN_TD_SHIFT               30
#define DMA_CHAN_CTRL_RX_GLOBAL_CHAN_TD_MASK                (1 << DMA_CHAN_CTRL_RX_GLOBAL_CHAN_TD_SHIFT)
#define DMA_CHAN_CTRL_RX_GLOBAL_SOP_OFFSET_SHIFT            16
#define DMA_CHAN_CTRL_RX_GLOBAL_SOP_OFFSET_MASK             (0xFF << DMA_CHAN_CTRL_RX_GLOBAL_SOP_OFFSET_SHIFT)
#define DMA_CHAN_CTRL_RX_GLOBAL_DEF_DESC_SHIFT              14
#define DMA_CHAN_CTRL_RX_GLOBAL_DEF_DESC_MASK               (0x3 << DMA_CHAN_CTRL_RX_GLOBAL_DEF_DESC_SHIFT)
#define DMA_CHAN_CTRL_RX_GLOBAL_RXCOMP_QMGR_SHIFT           12
#define DMA_CHAN_CTRL_RX_GLOBAL_RXCOMP_QMGR_MASK            (0x3 << DMA_CHAN_CTRL_RX_GLOBAL_RXCOMP_QMGR_SHIFT)
#define DMA_CHAN_CTRL_RX_GLOBAL_RXCOMP_QNUM_SHIFT           0
#define DMA_CHAN_CTRL_RX_GLOBAL_RXCOMP_QNUM_MASK            (0xFFF << DMA_CHAN_CTRL_RX_GLOBAL_RXCOMP_QNUM_SHIFT)

#define DMA_CHAN_CTRL_RX_HOSTPKT_FDB0_QNUM_SHIFT            0
#define DMA_CHAN_CTRL_RX_HOSTPKT_FDB0_QNUM_MASK             (0xFFF << DMA_CHAN_CTRL_RX_HOSTPKT_FDB0_QNUM_SHIFT)
#define DMA_CHAN_CTRL_RX_HOSTPKT_FDB0_QMGR_SHIFT            12
#define DMA_CHAN_CTRL_RX_HOSTPKT_FDB0_QMGR_MASK             (0x3 << DMA_CHAN_CTRL_RX_HOSTPKT_FDB0_QMGR_SHIFT)
#define DMA_CHAN_CTRL_RX_HOSTPKT_FDB1_QNUM_SHIFT            16
#define DMA_CHAN_CTRL_RX_HOSTPKT_FDB1_QNUM_MASK             (0xFFF << DMA_CHAN_CTRL_RX_HOSTPKT_FDB1_QNUM_SHIFT)
#define DMA_CHAN_CTRL_RX_HOSTPKT_FDB1_QMGR_SHIFT            28
#define DMA_CHAN_CTRL_RX_HOSTPKT_FDB1_QMGR_MASK             (0x3 << DMA_CHAN_CTRL_RX_HOSTPKT_FDB1_QMGR_SHIFT)
#define DMA_CHAN_CTRL_RX_HOSTPKT_FDB2_QNUM_SHIFT            0
#define DMA_CHAN_CTRL_RX_HOSTPKT_FDB2_QNUM_MASK             (0xFFF << DMA_CHAN_CTRL_RX_HOSTPKT_FDB2_QNUM_SHIFT)
#define DMA_CHAN_CTRL_RX_HOSTPKT_FDB2_QMGR_SHIFT            12
#define DMA_CHAN_CTRL_RX_HOSTPKT_FDB2_QMGR_MASK             (0x3 << DMA_CHAN_CTRL_RX_HOSTPKT_FDB2_QMGR_SHIFT)
#define DMA_CHAN_CTRL_RX_HOSTPKT_FDB3_QNUM_SHIFT            16
#define DMA_CHAN_CTRL_RX_HOSTPKT_FDB3_QNUM_MASK             (0xFFF << DMA_CHAN_CTRL_RX_HOSTPKT_FDB3_QNUM_SHIFT)
#define DMA_CHAN_CTRL_RX_HOSTPKT_FDB3_QMGR_SHIFT            28
#define DMA_CHAN_CTRL_RX_HOSTPKT_FDB3_QMGR_MASK             (0x3 << DMA_CHAN_CTRL_RX_HOSTPKT_FDB3_QMGR_SHIFT)

#define DMA_CHAN_CTRL_RX_EMBEDPKT_FBP0_PNUM_SHIFT           0
#define DMA_CHAN_CTRL_RX_EMBEDPKT_FBP0_PNUM_MASK            (0x1F << DMA_CHAN_CTRL_RX_EMBEDPKT_FBP0_PNUM_SHIFT)
#define DMA_CHAN_CTRL_RX_EMBEDPKT_FBP0_BMGR_SHIFT           6
#define DMA_CHAN_CTRL_RX_EMBEDPKT_FBP0_BMGR_MASK            (0x3 << DMA_CHAN_CTRL_RX_EMBEDPKT_FBP0_BMGR_SHIFT)
#define DMA_CHAN_CTRL_RX_EMBEDPKT_FBP1_PNUM_SHIFT           8
#define DMA_CHAN_CTRL_RX_EMBEDPKT_FBP1_PNUM_MASK            (0x1F << DMA_CHAN_CTRL_RX_EMBEDPKT_FBP1_PNUM_SHIFT)
#define DMA_CHAN_CTRL_RX_EMBEDPKT_FBP1_BMGR_SHIFT           14
#define DMA_CHAN_CTRL_RX_EMBEDPKT_FBP1_BMGR_MASK            (0x3 << DMA_CHAN_CTRL_RX_EMBEDPKT_FBP1_BMGR_SHIFT)
#define DMA_CHAN_CTRL_RX_EMBEDPKT_FBP2_PNUM_SHIFT           16
#define DMA_CHAN_CTRL_RX_EMBEDPKT_FBP2_PNUM_MASK            (0x1F << DMA_CHAN_CTRL_RX_EMBEDPKT_FBP2_PNUM_SHIFT)
#define DMA_CHAN_CTRL_RX_EMBEDPKT_FBP2_BMGR_SHIFT           22
#define DMA_CHAN_CTRL_RX_EMBEDPKT_FBP2_BMGR_MASK            (0x3 << DMA_CHAN_CTRL_RX_EMBEDPKT_FBP2_BMGR_SHIFT)
#define DMA_CHAN_CTRL_RX_EMBEDPKT_FBP3_PNUM_SHIFT           24
#define DMA_CHAN_CTRL_RX_EMBEDPKT_FBP3_PNUM_MASK            (0x1F << DMA_CHAN_CTRL_RX_EMBEDPKT_FBP3_PNUM_SHIFT)
#define DMA_CHAN_CTRL_RX_EMBEDPKT_FBP3_BMGR_SHIFT           30
#define DMA_CHAN_CTRL_RX_EMBEDPKT_FBP3_BMGR_MASK            (0x3 << DMA_CHAN_CTRL_RX_EMBEDPKT_FBP3_BMGR_SHIFT)

#define DMA_CHAN_CTRL_RX_EMBEDPKT_FDQ_QNUM_SHIFT            0
#define DMA_CHAN_CTRL_RX_EMBEDPKT_FDQ_QNUM_MASK             (0xFFF << DMA_CHAN_CTRL_RX_EMBEDPKT_FDQ_QNUM_SHIFT)
#define DMA_CHAN_CTRL_RX_EMBEDPKT_FDQ_QMGR_SHIFT            12
#define DMA_CHAN_CTRL_RX_EMBEDPKT_FDQ_QMGR_MASK             (0x3 << DMA_CHAN_CTRL_RX_EMBEDPKT_FDQ_QMGR_SHIFT)
#define DMA_CHAN_CTRL_RX_EMBEDPKT_SOP_SLOT_SHIFT            16
#define DMA_CHAN_CTRL_RX_EMBEDPKT_SOP_SLOT_MASK             (0x7 << DMA_CHAN_CTRL_RX_EMBEDPKT_SOP_SLOT_SHIFT)
#define DMA_CHAN_CTRL_RX_EMBEDPKT_NUM_SLOT_SHIFT            24
#define DMA_CHAN_CTRL_RX_EMBEDPKT_NUM_SLOT_MASK             (0x7 << DMA_CHAN_CTRL_RX_EMBEDPKT_NUM_SLOT_SHIFT)

#define DMA_CHAN_CTRL_RX_MONOLITHICPKT_FDQ_QNUM_SHIFT       0
#define DMA_CHAN_CTRL_RX_MONOLITHICPKT_FDQ_QNUM_MASK        (0xFFF << DMA_CHAN_CTRL_RX_MONOLITHICPKT_FDQ_QNUM_SHIFT)
#define DMA_CHAN_CTRL_RX_MONOLITHICPKT_FDQ_QMGR_SHIFT       12
#define DMA_CHAN_CTRL_RX_MONOLITHICPKT_FDQ_QMGR_MASK        (0x3 << DMA_CHAN_CTRL_RX_MONOLITHICPKT_FDQ_QMGR_SHIFT)
#define DMA_CHAN_CTRL_RX_MONOLITHICPKT_SOP_OFFSET_SHIFT     16
#define DMA_CHAN_CTRL_RX_MONOLITHICPKT_SOP_OFFSET_MASK      (0xFF << DMA_CHAN_CTRL_RX_MONOLITHICPKT_SOP_OFFSET_SHIFT)

#define DMA_SCHEDULER_ENABLE_BIT               (1<<31)

#define DOCSIS_DMA_TX_SCHEDULER_CONTROL_REG_ADDR               0x0006A000
#define DOCSIS_DMA_RX_SCHEDULER_CONTROL_REG_ADDR               0x0006C000

/* There are two DMA - one for 8 RX channels and 1 for 1 TX channel */
#define DOCSIS_DMA_RX_CH_CTRL_REG_BASE_ADDR                 0x00073000
#define DOCSIS_DMA_TX_CH_CTRL_REG_BASE_ADDR                 0x00071000

#define DOCSIS_DMA_TX_CTRL_GLOBAL_REG_BASE_ADDR             0x00072000
#define DOCSIS_DMA_RX_CH_GLOBAL_CTRL_REG_BASE_ADDR          0x00074000

#endif /* __DOCSIS_CPPI_REGS_H__ */
