/*
 *
 * baby_puma_driver.h
 * Description:
 * declaration of functions and types used to control the Baby Puma chip.
 *
 * Copyright (C) 2008 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifndef _HAL_BABY_PUMA_DRIVER_H_
#define _HAL_BABY_PUMA_DRIVER_H_

#include <asm/ioctl.h>
#include "sys_ptypes.h"
#include "soc_modules.h"
#include "soc_interface_driver.h"

/**************************************************************************/
/*      INTERFACE  Defines and Structs                                    */
/**************************************************************************/
#define BABY_PUMA_DRIVER_DEV_NAME  SOC_INTERFACE_DRIVER_DEV_NAME

/*****************************************/
/* Definitions for Baby Puma Driver      */
/*****************************************/

/* IOCTL commands:

   If you are adding new ioctl's to the kernel, you should use the _IO
   macros defined in <linux/ioctl.h> _IO macros are used to create ioctl numbers:

	_IO(type, nr)         - an ioctl with no parameter. 
   _IOW(type, nr, size)  - an ioctl with write parameters (copy_from_user), kernel would actually read data from user space 
   _IOR(type, nr, size)  - an ioctl with read parameters (copy_to_user), kernel would actually write data to user space 
   _IOWR(type, nr, size) - an ioctl with both write and read parameters 

   'Write' and 'read' are from the user's point of view, just like the
	system calls 'write' and 'read'.  For example, a SET_FOO ioctl would
	be _IOW, although the kernel would actually read data from user space;
	a GET_FOO ioctl would be _IOR, although the kernel would actually write
	data to user space.

	The first argument to _IO, _IOW, _IOR, or _IOWR is an identifying letter
	or number from the SoC_ModuleIds_e enum located in this file. 

	The second argument to _IO, _IOW, _IOR, or _IOWR is a sequence number
	to distinguish ioctls from each other.  

   The third argument to _IOW, _IOR, or _IOWR is the type of the data going
   into the kernel or coming out of the kernel (e.g.  'int' or 'struct foo').

   NOTE!  Do NOT use sizeof(arg) as the third argument as this results in 
   your ioctl thinking it passes an argument of type size_t.

*/
#define	BABY_PUMA_WRITE_PHY_REG    _IOW(SOC_BABY_PUMA_MODULE_ID, 1, RemoteRegInfo_t)
#define	BABY_PUMA_READ_PHY_REG     _IOWR(SOC_BABY_PUMA_MODULE_ID,2, RemoteRegInfo_t)
#define	BABY_PUMA_DRIVER_INIT      _IOW(SOC_BABY_PUMA_MODULE_ID, 3, Uint32)
#define	BABY_PUMA_GET_SOC_ADDR     _IOR(SOC_BABY_PUMA_MODULE_ID, 4, Uint32)


/* BABY_PUMA_IOCTL_MAXNR: In case of adding new Baby-Puma IOCTL commands this number must be update */
#define BABY_PUMA_IOCTL_MAXNR 4

/**************************************************************************/
/*! \fn void babyPumaRegisterSoCModule (SoCDriverOperations_t **mpeg_out_operations)
 **************************************************************************
 *  \brief Register the Baby-Puma driver function pointers object.
 *  \param[in] **baby_puma_operations_out - pointer to Baby-Puma operations.
 *  \return none.
 **************************************************************************/
#ifdef __KERNEL__
void babyPumaRegisterSoCModule (SoCDriverOperations_t **baby_puma_operations_out);

/**************************************************************************/
/*! \fn Uint32 babyPumaGetSoCAddr (void)
 **************************************************************************
 *  \brief Get the start address of the local VLYNQ window that represents 
 *  \brief the remote (Baby-Puma) SoC Addr.
 *  \return Local VLYNQ window that represents the remote SoC Addr.
 **************************************************************************/
Uint32 babyPumaGetSoCAddr (void);

/**************************************************************************/
/*! \fn Uint32 babyPumaGetMpegOutAddr (void)
 **************************************************************************
 *  \brief Get the start address of the local VLYNQ window that represents 
 *  \brief the remote (Baby-Puma) MPEG-Out Addr.
 *  \return Local VLYNQ window that represents the remote MPEG-Out Addr.
 **************************************************************************/
Uint32 babyPumaGetMpegOutAddr (void);

/**************************************************************************/
/*! \fn Uint32 babyPumaGetDocsisAddr (void)
 **************************************************************************
 *  \brief Get the start address of the local VLYNQ window that represents 
 *  \brief the remote (Baby-Puma) DOCSIS Addr.
 *  \return Local VLYNQ window that represents the remote DOCSIS Addr.
 **************************************************************************/
Uint32 babyPumaGetDocsisAddr (void);

/**************************************************************************/
/*! \fn int babyPumaIsInitialized ()
 **************************************************************************
 *  \brief Gets the babyPuma_driver_init variable. 
 **************************************************************************/
Uint32 babyPumaIsInitialized (void);

#endif /* __KERNEL__ */

#endif  /* _HAL_MPEG_SOC_DRIVER_H_ */
