#include <pal.h>
#include <asm/arch/puma5/puma5_cppi.h>
#include <pal_cppi41pvt.h>

#include "secondary_puma_driver.h"
#include "soc_interface_driver.h"


/* Definitions */
#define PUMA5_SR_BASE_PHYS                          (0x03000000)
#define PUMA5_DOCSIS_BASE_PHYS                      (0x09000000)
#define PUMA5_RAM_BASE_PHYS                         (AVALANCHE_SDRAM_BASE)

#define SECONDARY_BOOT_CFG_DELAY_MSEC                1

static  Uint32 SECONDARY_TRANSLATE_CPPI (Uint32 vaddr, Uint32 base)
{
    Uint32 tmpAddr = IO_VIRT2PHY(vaddr);
    Uint32 effectiveBase;

    if (IO_PHY2VIRT(0) > vaddr)
    {
        printk(" %s : RAM case : vaddr=0x%08X base=0x%08X\n", __FUNCTION__, vaddr, base);
        effectiveBase = PUMA5_RAM_BASE_PHYS;
        tmpAddr = vaddr;
    }
    else
    if (PUMA5_DOCSIS_BASE_PHYS <= tmpAddr)
    {
        effectiveBase = PUMA5_DOCSIS_BASE_PHYS;
    }
    else
    {
        effectiveBase = PUMA5_SR_BASE_PHYS;
    }

    return (tmpAddr - effectiveBase + base);
}



#define SECONDARY_SR0_PSC_OFFSET                 (0x21A60) /* 0x08621a60 */
#define SECONDARY_SR0_PSC_REG                    GET_SOC_REG_ADDR( SOC_DEVICE_REMOTE, SECONDARY_SR0_PSC_OFFSET )
#define SECONDARY_SR1_PSC_OFFSET                 (0x21A64) /* 0x08621a64 */
#define SECONDARY_SR1_PSC_REG                    GET_SOC_REG_ADDR( SOC_DEVICE_REMOTE, SECONDARY_SR1_PSC_OFFSET )
#define SECONDARY_SR2_PSC_OFFSET                 (0x21A68) /* 0x08621a68 */
#define SECONDARY_SR2_PSC_REG                    GET_SOC_REG_ADDR( SOC_DEVICE_REMOTE, SECONDARY_SR2_PSC_OFFSET )
#define SECONDARY_SR3_PSC_OFFSET                 (0x21A6C) /* 0x08621a6c */
#define SECONDARY_SR3_PSC_REG                    GET_SOC_REG_ADDR( SOC_DEVICE_REMOTE, SECONDARY_SR3_PSC_OFFSET )
#define SECONDARY_SR4_PSC_OFFSET                 (0x21A70) /* 0x08621a70 */
#define SECONDARY_SR4_PSC_REG                    GET_SOC_REG_ADDR( SOC_DEVICE_REMOTE, SECONDARY_SR4_PSC_OFFSET )
#define SECONDARY_SR5_PSC_OFFSET                 (0x21A74) /* 0x08621a74 */
#define SECONDARY_SR5_PSC_REG                    GET_SOC_REG_ADDR( SOC_DEVICE_REMOTE, SECONDARY_SR5_PSC_OFFSET )
#define SECONDARY_SR6_PSC_OFFSET                 (0x21A78) /* 0x08621a78 */
#define SECONDARY_SR6_PSC_REG                    GET_SOC_REG_ADDR( SOC_DEVICE_REMOTE, SECONDARY_SR6_PSC_OFFSET )
#define SECONDARY_SR7_PSC_OFFSET                 (0x21A7C) /* 0x08621a7C */
#define SECONDARY_SR7_PSC_REG                    GET_SOC_REG_ADDR( SOC_DEVICE_REMOTE, SECONDARY_SR7_PSC_OFFSET )
#define SECONDARY_DDR_PHY_PSC_OFFSET             (0x21A10) /* 0x08621a10 */
#define SECONDARY_DDR_PHY_PSC_REG                GET_SOC_REG_ADDR( SOC_DEVICE_REMOTE, SECONDARY_DDR_PHY_PSC_OFFSET )
#define SECONDARY_DDR_CTL_PSC_OFFSET             (0x21A14) /* 0x08621a14 */
#define SECONDARY_DDR_CTL_PSC_REG                GET_SOC_REG_ADDR( SOC_DEVICE_REMOTE, SECONDARY_DDR_CTL_PSC_OFFSET )
#define SECONDARY_PERF_MON_PSC_OFFSET             (0x21A28) /* 0x08621a28 */
#define SECONDARY_PERF_MON_PSC_REG                GET_SOC_REG_ADDR( SOC_DEVICE_REMOTE, SECONDARY_PERF_MON_PSC_OFFSET )
#define SECONDARY_SR_PSC_VAL                     (0x103) 
#define SECONDARY_PSC_GO_OFFSET                  (0x21120) /* 0x08621120 */
#define SECONDARY_PSC_GO_REG                     GET_SOC_REG_ADDR( SOC_DEVICE_REMOTE, PUMA5_PSC_GO_OFFSET )
#define SECONDARY_PSC_GO_VAL                     (0x00000001)


/**************************************************************************/
/*! \fn static void secondaryPuma_setupRecycleInfra (PAL_Handle hnd)
 **************************************************************************
 *  \brief Setup infrastructure DMA for recycling use resources.
 *  \return none.
 **************************************************************************/
static void secondaryPuma_setupRecycleInfra (PAL_Handle hnd)
{
    Cppi4TxChInitCfg txCh;
    Cppi4RxChInitCfg rxCh;
    PAL_Cppi4TxChHnd cppi4TxChHnd;
    PAL_Cppi4RxChHnd cppi4RxChHnd;
    Cppi4Queue tmpQ;

    tmpQ.qMgr = CNI_CPPI4x_FBD_QMGR;
    tmpQ.qNum = CNI_CPPI4x_FBD_QNUM( PAL_CPPI4x_PRTY_LOW );
    PAL_cppi4QueueOpen (hnd, tmpQ);

    /* Set up Rx channel */
    rxCh.chNum              = CNI_CPPI4x_DOC2HOST_PROXY_CHNUM(0); 
    rxCh.dmaNum             = PAL_CPPI41_DMA_BLOCK1;
    rxCh.sopOffset          = 0;
    rxCh.retryOnStarvation  = 0;
    rxCh.rxCompQueue.qMgr   = CNI_CPPI4x_RX_QMGR;
    rxCh.rxCompQueue.qNum   = CNI_CPPI4x_RX_QNUM( PAL_CPPI4x_PRTY_LOW );

    rxCh.defDescType        = CPPI41_DESC_TYPE_EMBEDDED;
    rxCh.u.embeddedPktCfg.fdQueue.qMgr = CNI_CPPI4x_FBD_QMGR;
    rxCh.u.embeddedPktCfg.fdQueue.qNum = CNI_CPPI4x_FBD_QNUM( PAL_CPPI4x_PRTY_LOW );
    rxCh.u.embeddedPktCfg.numBufSlot = (EMSLOTCNT-1);
    rxCh.u.embeddedPktCfg.sopSlotNum = 0;
    rxCh.u.embeddedPktCfg.fBufPool[0].bMgr  = BUF_POOL_MGR0;
    rxCh.u.embeddedPktCfg.fBufPool[0].bPool = BMGR0_POOL12;
    rxCh.u.embeddedPktCfg.fBufPool[1].bMgr  = BUF_POOL_MGR0;
    rxCh.u.embeddedPktCfg.fBufPool[1].bPool = BMGR0_POOL12;
    rxCh.u.embeddedPktCfg.fBufPool[2].bMgr  = BUF_POOL_MGR0;
    rxCh.u.embeddedPktCfg.fBufPool[2].bPool = BMGR0_POOL12;
    rxCh.u.embeddedPktCfg.fBufPool[3].bMgr  = BUF_POOL_MGR0;
    rxCh.u.embeddedPktCfg.fBufPool[3].bPool = BMGR0_POOL12;

//    rxCh.defDescType        = CPPI41_DESC_TYPE_HOST;
//    rxCh.u.hostPktCfg.fdbQueue[0].qMgr = CNI_CPPI4x_FBD_QMGR;                       
//    rxCh.u.hostPktCfg.fdbQueue[0].qNum = CNI_CPPI4x_FBD_QNUM( PAL_CPPI4x_PRTY_LOW );
//    rxCh.u.hostPktCfg.fdbQueue[1].qMgr = CNI_CPPI4x_FBD_QMGR;                       
//    rxCh.u.hostPktCfg.fdbQueue[1].qNum = CNI_CPPI4x_FBD_QNUM( PAL_CPPI4x_PRTY_LOW );
//    rxCh.u.hostPktCfg.fdbQueue[2].qMgr = CNI_CPPI4x_FBD_QMGR;                       
//    rxCh.u.hostPktCfg.fdbQueue[2].qNum = CNI_CPPI4x_FBD_QNUM( PAL_CPPI4x_PRTY_LOW );
//    rxCh.u.hostPktCfg.fdbQueue[3].qMgr = CNI_CPPI4x_FBD_QMGR;                       
//    rxCh.u.hostPktCfg.fdbQueue[3].qNum = CNI_CPPI4x_FBD_QNUM( PAL_CPPI4x_PRTY_LOW );

    cppi4RxChHnd        = PAL_cppi4RxChOpen(hnd, &rxCh, NULL);
    
    /* Set up Tx channel */
    txCh.chNum          = CNI_CPPI4x_DOC2HOST_PROXY_CHNUM(0);
    txCh.dmaNum         = PAL_CPPI41_DMA_BLOCK1;
    txCh.tdQueue.qMgr   = DMA1_CPPI4x_FTD_QMGR;
    txCh.tdQueue.qNum   = DMA1_CPPI4x_FTD_QNUM;

    cppi4TxChHnd        = PAL_cppi4TxChOpen(hnd, &txCh, NULL);

    if (!cppi4TxChHnd || !cppi4RxChHnd)
    {
        printk ("%s: infra channel setup failed for channel %d\n",
                __FUNCTION__, CNI_CPPI4x_DOC2HOST_PROXY_CHNUM(0) );   
        return;
    }
    
    /* Enable Tx-Rx channels */
    PAL_cppi4EnableRxChannel (cppi4RxChHnd, NULL);
    PAL_cppi4EnableTxChannel (cppi4TxChHnd, NULL);

    return;
}



/**************************************************************************/
/*! \fn static void secondaryPumaCppiConfigTranslation       (void)
 **************************************************************************
 *  \brief Perform the required translations on the CPPI config structure.
 *  \return none.
 **************************************************************************/
static void secondaryPumaCppiConfigTranslation (Cppi4InitCfg*   cfg, Uint32 cppiBase)
{
//    cppiBase = secondaryPumaGetDynamic1Addr();
//    docsisBase = secondaryPumaGetDocsisAddr();  

    /************************************************************************/
    /*                                                                      */
    /************************************************************************/
    if (cfg->dmaBlock[PAL_CPPI41_DMA_BLOCK0].globalCtrlBase)
    {
        cfg->dmaBlock[PAL_CPPI41_DMA_BLOCK0].globalCtrlBase   = (Ptr)SECONDARY_TRANSLATE_CPPI((Uint32)cfg->dmaBlock[PAL_CPPI41_DMA_BLOCK0].globalCtrlBase ,     cppiBase);
        cfg->dmaBlock[PAL_CPPI41_DMA_BLOCK0].chCtrlStatusBase = (Ptr)SECONDARY_TRANSLATE_CPPI((Uint32)cfg->dmaBlock[PAL_CPPI41_DMA_BLOCK0].chCtrlStatusBase ,   cppiBase);
        cfg->dmaBlock[PAL_CPPI41_DMA_BLOCK0].schedCtrlBase    = (Ptr)SECONDARY_TRANSLATE_CPPI((Uint32)cfg->dmaBlock[PAL_CPPI41_DMA_BLOCK0].schedCtrlBase ,      cppiBase);
        cfg->dmaBlock[PAL_CPPI41_DMA_BLOCK0].schedTableBase   = (Ptr)SECONDARY_TRANSLATE_CPPI((Uint32)cfg->dmaBlock[PAL_CPPI41_DMA_BLOCK0].schedTableBase ,     cppiBase);
    }

    if (cfg->dmaBlock[PAL_CPPI41_DMA_BLOCK1].globalCtrlBase)
    {
        cfg->dmaBlock[PAL_CPPI41_DMA_BLOCK1].globalCtrlBase   = (Ptr)SECONDARY_TRANSLATE_CPPI((Uint32)cfg->dmaBlock[PAL_CPPI41_DMA_BLOCK1].globalCtrlBase ,     cppiBase);
        cfg->dmaBlock[PAL_CPPI41_DMA_BLOCK1].chCtrlStatusBase = (Ptr)SECONDARY_TRANSLATE_CPPI((Uint32)cfg->dmaBlock[PAL_CPPI41_DMA_BLOCK1].chCtrlStatusBase ,   cppiBase);
        cfg->dmaBlock[PAL_CPPI41_DMA_BLOCK1].schedCtrlBase    = (Ptr)SECONDARY_TRANSLATE_CPPI((Uint32)cfg->dmaBlock[PAL_CPPI41_DMA_BLOCK1].schedCtrlBase ,      cppiBase);
        cfg->dmaBlock[PAL_CPPI41_DMA_BLOCK1].schedTableBase   = (Ptr)SECONDARY_TRANSLATE_CPPI((Uint32)cfg->dmaBlock[PAL_CPPI41_DMA_BLOCK1].schedTableBase ,     cppiBase);
    }

    /************************************************************************/
    /*                                                                      */
    /************************************************************************/
    if (cfg->bufMgrBase[ PAL_CPPI41_BUF_MGR_PARTITION_SR ])
    {
        cfg->bufMgrBase[ PAL_CPPI41_BUF_MGR_PARTITION_SR ] = 
            (Ptr)SECONDARY_TRANSLATE_CPPI((Uint32)cfg->bufMgrBase[ PAL_CPPI41_BUF_MGR_PARTITION_SR     ],  cppiBase);
    }

    if (cfg->bufMgrBase[ PAL_CPPI41_BUF_MGR_PARTITION_DOCSIS ])
    {
        cfg->bufMgrBase[ PAL_CPPI41_BUF_MGR_PARTITION_DOCSIS ] = 
            (Ptr)SECONDARY_TRANSLATE_CPPI((Uint32)cfg->bufMgrBase[ PAL_CPPI41_BUF_MGR_PARTITION_DOCSIS ],  cppiBase);
    }


    /************************************************************************/
    /*                                                                      */
    /************************************************************************/
    if (cfg->queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].queueMgrRgnBase)
    {
        cfg->queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].queueMgrRgnBase    = (Ptr) SECONDARY_TRANSLATE_CPPI((Uint32)cfg->queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].queueMgrRgnBase,    cppiBase);
        cfg->queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descMemRgnBase     = (Ptr) SECONDARY_TRANSLATE_CPPI((Uint32)cfg->queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descMemRgnBase,     cppiBase);
        cfg->queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].queueMgmtRgnBase   = (Ptr) SECONDARY_TRANSLATE_CPPI((Uint32)cfg->queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].queueMgmtRgnBase,   cppiBase);
		if (cfg->queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].queueProxyRgnBase)
        {
            cfg->queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].queueProxyRgnBase  = (Ptr) SECONDARY_TRANSLATE_CPPI((Uint32)cfg->queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].queueProxyRgnBase,  cppiBase);
        }
        cfg->queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].queueStatusRgnBase = (Ptr) SECONDARY_TRANSLATE_CPPI((Uint32)cfg->queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].queueStatusRgnBase, cppiBase);
    }

    if (cfg->queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_DOCSIS].queueMgrRgnBase)
    {
        cfg->queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_DOCSIS].queueMgrRgnBase    = (Ptr) SECONDARY_TRANSLATE_CPPI((Uint32)cfg->queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_DOCSIS].queueMgrRgnBase,    cppiBase);
        cfg->queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_DOCSIS].descMemRgnBase     = (Ptr) SECONDARY_TRANSLATE_CPPI((Uint32)cfg->queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_DOCSIS].descMemRgnBase,     cppiBase);
        cfg->queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_DOCSIS].queueMgmtRgnBase   = (Ptr) SECONDARY_TRANSLATE_CPPI((Uint32)cfg->queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_DOCSIS].queueMgmtRgnBase,   cppiBase);
        if (cfg->queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_DOCSIS].queueProxyRgnBase)
        {
            cfg->queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_DOCSIS].queueProxyRgnBase  = (Ptr) SECONDARY_TRANSLATE_CPPI((Uint32)cfg->queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_DOCSIS].queueProxyRgnBase,  cppiBase);
        }
        cfg->queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_DOCSIS].queueStatusRgnBase = (Ptr) SECONDARY_TRANSLATE_CPPI((Uint32)cfg->queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_DOCSIS].queueStatusRgnBase, cppiBase);
    }

    if (cfg->apdspInfo.firmwareSize)
    {
        cfg->apdspInfo.iRamBase     = (Ptr) SECONDARY_TRANSLATE_CPPI( (Uint32)cfg->apdspInfo.iRamBase,      cppiBase );
        cfg->apdspInfo.pdspCmdBase  = (Ptr) SECONDARY_TRANSLATE_CPPI( (Uint32)cfg->apdspInfo.pdspCmdBase,   cppiBase );
        cfg->apdspInfo.pdspCtrlBase = (Ptr) SECONDARY_TRANSLATE_CPPI( (Uint32)cfg->apdspInfo.pdspCtrlBase,  cppiBase );
    }
}


/**************************************************************************/
/*! \fn static void secondaryPumaCppiPsc()
 **************************************************************************
 *  \brief take the relevant modules out of reset using LPSC
 *  \param none
 *  \return none.
 **************************************************************************/
static void secondaryPumaCppiPsc()
{
    /* SOC_DEVICE_REMOTE SECONDARY_SR0_PSC_REG*/

   printk(KERN_INFO "Kick the BootCfg lock \n");
   socWriteRegister((PUMA5_BOOTCFG_KICK_0_REG( SOC_DEVICE_REMOTE )), PUMA5_BOOTCFG_KICK_0_VAL, True);
   socWriteRegister((PUMA5_BOOTCFG_KICK_1_REG( SOC_DEVICE_REMOTE )), PUMA5_BOOTCFG_KICK_1_VAL, True);
   
   /* Add delay */
   PAL_osWaitMsecs(SECONDARY_BOOT_CFG_DELAY_MSEC);
   
   printk("Configure SR/CPPI PSC \n");
   socWriteRegister(SECONDARY_SR0_PSC_REG, 0, True);
   socWriteRegister(SECONDARY_PSC_GO_REG, SECONDARY_PSC_GO_VAL, True);
   socWriteRegister(SECONDARY_SR1_PSC_REG, 0, True);
   socWriteRegister(SECONDARY_PSC_GO_REG, SECONDARY_PSC_GO_VAL, True);
   socWriteRegister(SECONDARY_SR2_PSC_REG, 0, True);
   socWriteRegister(SECONDARY_PSC_GO_REG, SECONDARY_PSC_GO_VAL, True);
   socWriteRegister(SECONDARY_SR3_PSC_REG, 0, True);
   socWriteRegister(SECONDARY_PSC_GO_REG, SECONDARY_PSC_GO_VAL, True);
   socWriteRegister(SECONDARY_SR4_PSC_REG, 0, True);
   socWriteRegister(SECONDARY_PSC_GO_REG, SECONDARY_PSC_GO_VAL, True);
   socWriteRegister(SECONDARY_SR5_PSC_REG, 0, True);
   socWriteRegister(SECONDARY_PSC_GO_REG, SECONDARY_PSC_GO_VAL, True);
   socWriteRegister(SECONDARY_SR6_PSC_REG, 0, True);
   socWriteRegister(SECONDARY_PSC_GO_REG, SECONDARY_PSC_GO_VAL, True);
   socWriteRegister(SECONDARY_SR7_PSC_REG, 0, True);
   socWriteRegister(SECONDARY_PSC_GO_REG, SECONDARY_PSC_GO_VAL, True);

   PAL_osWaitMsecs(100);
   
   socWriteRegister(SECONDARY_SR0_PSC_REG, SECONDARY_SR_PSC_VAL, True);
   socWriteRegister(SECONDARY_PSC_GO_REG, SECONDARY_PSC_GO_VAL, True);
   socWriteRegister(SECONDARY_SR1_PSC_REG, SECONDARY_SR_PSC_VAL, True);
   socWriteRegister(SECONDARY_PSC_GO_REG, SECONDARY_PSC_GO_VAL, True);
   socWriteRegister(SECONDARY_SR2_PSC_REG, SECONDARY_SR_PSC_VAL, True);
   socWriteRegister(SECONDARY_PSC_GO_REG, SECONDARY_PSC_GO_VAL, True);
   socWriteRegister(SECONDARY_SR3_PSC_REG, SECONDARY_SR_PSC_VAL, True);
   socWriteRegister(SECONDARY_PSC_GO_REG, SECONDARY_PSC_GO_VAL, True);
   socWriteRegister(SECONDARY_SR4_PSC_REG, SECONDARY_SR_PSC_VAL, True);
   socWriteRegister(SECONDARY_PSC_GO_REG, SECONDARY_PSC_GO_VAL, True);
   socWriteRegister(SECONDARY_SR5_PSC_REG, SECONDARY_SR_PSC_VAL, True);
   socWriteRegister(SECONDARY_PSC_GO_REG, SECONDARY_PSC_GO_VAL, True);
   socWriteRegister(SECONDARY_SR6_PSC_REG, SECONDARY_SR_PSC_VAL, True);
   socWriteRegister(SECONDARY_PSC_GO_REG, SECONDARY_PSC_GO_VAL, True);
   socWriteRegister(SECONDARY_SR7_PSC_REG, SECONDARY_SR_PSC_VAL, True);
   socWriteRegister(SECONDARY_PSC_GO_REG, SECONDARY_PSC_GO_VAL, True);
   socWriteRegister(SECONDARY_DDR_PHY_PSC_REG, SECONDARY_SR_PSC_VAL, True);
   socWriteRegister(SECONDARY_PSC_GO_REG, SECONDARY_PSC_GO_VAL, True);
   socWriteRegister(SECONDARY_DDR_CTL_PSC_REG, SECONDARY_SR_PSC_VAL, True);
   socWriteRegister(SECONDARY_PSC_GO_REG, SECONDARY_PSC_GO_VAL, True);
   socWriteRegister(SECONDARY_PERF_MON_PSC_REG, SECONDARY_SR_PSC_VAL, True);
   socWriteRegister(SECONDARY_PSC_GO_REG, SECONDARY_PSC_GO_VAL, True);


   PAL_osWaitMsecs(SECONDARY_BOOT_CFG_DELAY_MSEC);

   return;
}

/**
 *  \brief PAL os Memory Allocation API. Allocated memory
 *  memory will be aligned to requested size. For smaller
 *  memory chunks (less than page size) use PAL_osMemAlloc
 */ 
static Uint32 gAllocStart = AVALANCHE_SDRAM_BASE;

Ptr secondaryPuma_memAllocSizeAligned( Uint32 numBytes )
{
    Uint32 order;
    Uint32 ret;
    
    printk ("  =======> 0x%08X  ...(0x%08x) ", gAllocStart, numBytes);

    /* find number of pages */        
    numBytes = (numBytes/PAGE_SIZE) + ((numBytes % PAGE_SIZE)?1:0);
    
    /* find allocation order */
    for(order = 0; (1 << order) < numBytes; order++);

    ret = gAllocStart;
    numBytes = (1 << order) * PAGE_SIZE;

    gAllocStart += numBytes;
    
    if ((AVALANCHE_SDRAM_BASE + 4096*PAGE_SIZE) < gAllocStart)
    {
        gAllocStart -= numBytes;
        ret = 0;
    }

    printk ("->(0x%08X) ... 0x%08X <=======\n",numBytes, ret);

    return (void*) ret;
}

     



/************************************************************************/
/*   ____                _               ____             _             */
/*  / ___|  ___  ___ ___(_) ___  _ __   |  _ \ ___  _   _| |_ ___ _ __  */
/*  \___ \ / _ \/ __/ __| |/ _ \| '_ \  | |_) / _ \| | | | __/ _ \ '__| */
/*   ___) |  __/\__ \__ \ | (_) | | | | |  _ < (_) | |_| | ||  __/ |    */
/*  |____/ \___||___/___/_|\___/|_| |_| |_| \_\___/ \__,_|\__\___|_|    */
/*                                                                      */
/*                         ___       _ _                                */
/*                        |_ _|_ __ (_) |_                              */
/*                         | || '_ \| | __|                             */
/*                         | || | | | | |_                              */
/*                        |___|_| |_|_|\__|                             */
/*                                                                      */
/************************************************************************/
Cppi4InitCfg cppi4InitCfgSecondarySR_g = 
{
    .resetLine              = 0,
    .appSpecificMemAllocFunc = secondaryPuma_memAllocSizeAligned,
    
    .dmaBlock[PAL_CPPI41_DMA_BLOCK0].globalCtrlBase     = (Ptr) AVALANCHE_NWSS_DMA0_GBLCFG_BASE,
    .dmaBlock[PAL_CPPI41_DMA_BLOCK0].chCtrlStatusBase   = (Ptr) AVALANCHE_NWSS_DMA0_CHNCFG_BASE,
    .dmaBlock[PAL_CPPI41_DMA_BLOCK0].schedCtrlBase      = (Ptr) AVALANCHE_NWSS_DMA0_SCHEDCFG_BASE,
    .dmaBlock[PAL_CPPI41_DMA_BLOCK0].schedTableBase     = (Ptr) AVALANCHE_NWSS_DMA0_SCHEDTBL_BASE,
    
    .dmaBlock[PAL_CPPI41_DMA_BLOCK0].tdFQueue.qMgr      = DMA0_CPPI4x_FTD_QMGR,
    .dmaBlock[PAL_CPPI41_DMA_BLOCK0].tdFQueue.qNum      = DMA0_CPPI4x_FTD_QNUM,
  
    .dmaBlock[PAL_CPPI41_DMA_BLOCK1].globalCtrlBase     = (Ptr) AVALANCHE_NWSS_DMA1_GBLCFG_BASE,
    .dmaBlock[PAL_CPPI41_DMA_BLOCK1].chCtrlStatusBase   = (Ptr) AVALANCHE_NWSS_DMA1_CHNCFG_BASE,
    .dmaBlock[PAL_CPPI41_DMA_BLOCK1].schedCtrlBase      = (Ptr) AVALANCHE_NWSS_DMA1_SCHEDCFG_BASE,
    .dmaBlock[PAL_CPPI41_DMA_BLOCK1].schedTableBase     = (Ptr) AVALANCHE_NWSS_DMA1_SCHEDTBL_BASE,

    .dmaBlock[PAL_CPPI41_DMA_BLOCK1].tdFQueue.qMgr      = DMA1_CPPI4x_FTD_QMGR,
    .dmaBlock[PAL_CPPI41_DMA_BLOCK1].tdFQueue.qNum      = DMA1_CPPI4x_FTD_QNUM,


    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].queueMgrRgnBase        = (Ptr) AVALANCHE_NWSS_QMGR_RGN_BASE, 
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descMemRgnBase         = (Ptr) AVALANCHE_NWSS_DESCMEM_RGN_BASE,     
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].queueMgmtRgnBase       = (Ptr) AVALANCHE_NWSS_QMGMT_RGN_BASE,
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].queueProxyRgnBase      = (Ptr) AVALANCHE_NWSS_QPROXY_0_RGN_BASE,
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].queueStatusRgnBase     = (Ptr) AVALANCHE_NWSS_QSTATUS_RGN_BASE,
    
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].LinkingRAM0Base        = IO_VIRT2PHY( AVALANCHE_NWSS_PACKET_RAM_BASE ), 
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].LinkingRAM0Size        = 7168, 
            /* Max pointers in this region could be 7K (assuming 32-bit pointers). */
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].LinkingRAM1Base        = 0,

    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descRegion[0].base     = (Ptr) 0x01119000,   /* Docsis MAC internal memory */
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descRegion[0].szDesc   = 128, 
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descRegion[0].numDesc  = 32,
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descRegion[0].isOnChip = 1,    

    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descRegion[1].base     = (Ptr) IO_VIRT2PHY( AVALANCHE_NWSS_ONCHIPDESC_BASE ),    /* Onchip */
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descRegion[1].szDesc   = PPFW_CPPI4x_BD_SIZE, 
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descRegion[1].numDesc  = PPFW_PREFETCH_DESC_COUNT + PPFW_REPLICA_DESC_COUNT,     /* Total BD Count: Prefetcher + Replication */
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descRegion[1].isOnChip = 1,    
    
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descRegion[2].base     = (Ptr) IO_VIRT2PHY( AVALANCHE_NWSS_APDSP_PREFBLK_BASE ), /* Prefetch */
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descRegion[2].szDesc   = PPFW_CPPI4x_BUF_BD_SIZE, 
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descRegion[2].numDesc  = PPFW_PREFETCH_BUFF_COUNT,
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descRegion[2].isOnChip = 1,

	/* Reserve region with following specifics for Embedded */
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descRegion[3].base     = 0,
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descRegion[3].szDesc   = 64,
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descRegion[3].numDesc  = CNI_RX_EMBEDDED_BD_NUM_LOW + CNI_RX_EMBEDDED_BD_NUM_HIGH /* !@@ Use as per PP system req */,

    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descRegion[4].base     = 0,
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descRegion[4].szDesc   = USB_CPPI4x_USB_BD_SIZE, 
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descRegion[4].numDesc  = USB_CPPI4x_MAX_USB_DESC, 

    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descRegion[5].base     = 0,
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descRegion[5].szDesc   = CPMAC_CPPI4x_TX_HOST_BD_SIZE, /* sizeof (Cppi4HostDesc) + alignment */
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descRegion[5].numDesc  = CPMAC_TX_HOST_BD_NUM,

    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descRegion[6].base     = 0,
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descRegion[6].szDesc   = CPMAC_CPPI4x_RX_HOST_BD_SIZE,
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descRegion[6].numDesc  = CPMAC_RX_HOST_BD_NUM,

    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descRegion[7].base     = 0,
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descRegion[7].szDesc   = 64, /* 64 byte sized */
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descRegion[7].numDesc  = 64,    

    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descRegion[8].base     = 0,
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descRegion[8].szDesc   = 64, /* 64 byte sized */
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descRegion[8].numDesc  = 64,    

    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descRegion[9].base     = 0,
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descRegion[9].szDesc   = 64, /* 64 byte sized */
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descRegion[9].numDesc  = 256, //512,    

    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descRegion[10].base    = 0,
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descRegion[10].szDesc  = sizeof(Cppi4TeardownDesc), /* 32 byte sized */
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descRegion[10].numDesc = PAL_CPPI41_NUM_TD_DESC,

    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descRegion[11].base    = 0,
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descRegion[11].szDesc  = sizeof(Cppi4TeardownDesc), /* 32 byte sized */
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descRegion[11].numDesc = PAL_CPPI41_NUM_TD_DESC,

    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descRegion[12].base    = 0,
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descRegion[12].szDesc  = CNI_BD_SIZE,
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descRegion[12].numDesc = CNI_BD_NUM,  
    
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descRegion[13].base    = 0,
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descRegion[13].szDesc  = PPFW_HOST_BD_SIZE,
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descRegion[13].numDesc = PPFW_HOST_BD_NUM,  
    
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descRegion[14].base     = 0,
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descRegion[14].szDesc   = MPEG_BD_SIZE,
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descRegion[14].numDesc  = DMAC_MPEG_RX_EMBEDDED_BD_NUM,
    
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descRegion[15].base     = 0,
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descRegion[15].szDesc   = MPEG_ENCAP_BD_SIZE,
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].descRegion[15].numDesc  = DMAC_MPEG_ENCAP_RX_EMBEDDED_BD_NUM,
    
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].basefdQNum             = PAL_CPPI41_FD_Q_BASE,
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_SR].basefdbQNum            = PAL_CPPI41_FBD_Q_BASE,    


    .bufMgrBase  [PAL_CPPI41_BUF_MGR_PARTITION_SR]                          = (CSL_BufMgr_RegsOvly) AVALANCHE_NWSS_BMGR_BASE,

    .dmaBlock[PAL_CPPI41_DMA_BLOCK0].schedTable.numEntries  = 2,
    .dmaBlock[PAL_CPPI41_DMA_BLOCK0].schedTable.entry = 
    { 
        PAL_CPPI41_DMA_CH_CONFIG( PAL_CPPI41_DMA_RX_CH, CPMAC_CPPI4x_RX_DMA_CHNUM(0) ),
        PAL_CPPI41_DMA_CH_CONFIG( PAL_CPPI41_DMA_TX_CH, CPMAC_CPPI4x_TX_DMA_CHNUM(0) ),
    },

    .dmaBlock[PAL_CPPI41_DMA_BLOCK1].schedTable.numEntries  = 35,
    .dmaBlock[PAL_CPPI41_DMA_BLOCK1].schedTable.entry = 
    {   
        PAL_CPPI41_DMA_CH_CONFIG( PAL_CPPI41_DMA_RX_CH, CPMAC_CPPI4x_ETH2HOST_PROXY_CHNUM(0) ),
        PAL_CPPI41_DMA_CH_CONFIG( PAL_CPPI41_DMA_TX_CH, CPMAC_CPPI4x_ETH2HOST_PROXY_CHNUM(0) ),
        PAL_CPPI41_DMA_CH_CONFIG( PAL_CPPI41_DMA_RX_CH, USB_CPPI4x_RX_DMA_CHNUM(0)  ),
        PAL_CPPI41_DMA_CH_CONFIG( PAL_CPPI41_DMA_TX_CH, USB_CPPI4x_TX_DMA_CHNUM(0)  ),
        PAL_CPPI41_DMA_CH_CONFIG( PAL_CPPI41_DMA_RX_CH, USB_CPPI4x_RX_DMA_CHNUM(1)  ),
        PAL_CPPI41_DMA_CH_CONFIG( PAL_CPPI41_DMA_TX_CH, USB_CPPI4x_TX_DMA_CHNUM(1)  ),
        PAL_CPPI41_DMA_CH_CONFIG( PAL_CPPI41_DMA_RX_CH, USB_CPPI4x_RX_DMA_CHNUM(2)  ),
        PAL_CPPI41_DMA_CH_CONFIG( PAL_CPPI41_DMA_TX_CH, USB_CPPI4x_TX_DMA_CHNUM(2)  ),
        PAL_CPPI41_DMA_CH_CONFIG( PAL_CPPI41_DMA_RX_CH, USB_CPPI4x_RX_DMA_CHNUM(3)  ), 
        PAL_CPPI41_DMA_CH_CONFIG( PAL_CPPI41_DMA_TX_CH, USB_CPPI4x_TX_DMA_CHNUM(3)  ),

        PAL_CPPI41_DMA_CH_CONFIG( PAL_CPPI41_DMA_RX_CH, CNI_CPPI4x_RX_DMA_CHNUM(0)  ),
        PAL_CPPI41_DMA_CH_CONFIG( PAL_CPPI41_DMA_RX_CH, CNI_CPPI4x_RX_DMA_CHNUM(1)  ),
        PAL_CPPI41_DMA_CH_CONFIG( PAL_CPPI41_DMA_RX_CH, CNI_CPPI4x_RX_DMA_CHNUM(2)  ),
        PAL_CPPI41_DMA_CH_CONFIG( PAL_CPPI41_DMA_RX_CH, MPEG_CPPI4x_RX_DMA_CHNUM    ),

        PAL_CPPI41_DMA_CH_CONFIG( PAL_CPPI41_DMA_TX_CH, MPEGOUT_CPPI4x_CHNUM        ),
        
        PAL_CPPI41_DMA_CH_CONFIG( PAL_CPPI41_DMA_TX_CH, CNI_CPPI4x_TX_DMA_CHNUM(0)  ),
        PAL_CPPI41_DMA_CH_CONFIG( PAL_CPPI41_DMA_TX_CH, CNI_CPPI4x_TX_DMA_CHNUM(1)  ),
        PAL_CPPI41_DMA_CH_CONFIG( PAL_CPPI41_DMA_TX_CH, CNI_CPPI4x_TX_DMA_CHNUM(2)  ),
        PAL_CPPI41_DMA_CH_CONFIG( PAL_CPPI41_DMA_TX_CH, CNI_CPPI4x_TX_DMA_CHNUM(3)  ),

        PAL_CPPI41_DMA_CH_CONFIG( PAL_CPPI41_DMA_RX_CH, CNI_CPPI4x_DOCSIS_TO_HOST_MGMT_CHNUM ),
        PAL_CPPI41_DMA_CH_CONFIG( PAL_CPPI41_DMA_TX_CH, CNI_CPPI4x_DOCSIS_TO_HOST_MGMT_CHNUM ),
        PAL_CPPI41_DMA_CH_CONFIG( PAL_CPPI41_DMA_RX_CH, CNI_CPPI4x_RX_CoP_DMA_CHNUM(0)  ),
        PAL_CPPI41_DMA_CH_CONFIG( PAL_CPPI41_DMA_TX_CH, CNI_CPPI4x_TX_CoP_DMA_CHNUM(0)  ),
        PAL_CPPI41_DMA_CH_CONFIG( PAL_CPPI41_DMA_RX_CH, CNI_CPPI4x_DOC2HOST_PROXY_CHNUM(0)  ),
        PAL_CPPI41_DMA_CH_CONFIG( PAL_CPPI41_DMA_TX_CH, CNI_CPPI4x_DOC2HOST_PROXY_CHNUM(0)  ),
        PAL_CPPI41_DMA_CH_CONFIG( PAL_CPPI41_DMA_RX_CH, CNI_CPPI4x_DOC2HOST_PROXY_CHNUM(1)  ),
        PAL_CPPI41_DMA_CH_CONFIG( PAL_CPPI41_DMA_TX_CH, CNI_CPPI4x_DOC2HOST_PROXY_CHNUM(1)  ),
        PAL_CPPI41_DMA_CH_CONFIG( PAL_CPPI41_DMA_RX_CH, CNI_CPPI4x_DOC2HOST_PROXY_CHNUM(2)  ),
        PAL_CPPI41_DMA_CH_CONFIG( PAL_CPPI41_DMA_TX_CH, CNI_CPPI4x_DOC2HOST_PROXY_CHNUM(2)  ),
		PAL_CPPI41_DMA_CH_CONFIG( PAL_CPPI41_DMA_RX_CH, USB_CPPI4x_USB2HOST_PROXY_CHNUM(0)      ),
        PAL_CPPI41_DMA_CH_CONFIG( PAL_CPPI41_DMA_TX_CH, USB_CPPI4x_USB2HOST_PROXY_CHNUM(0)      ),
        PAL_CPPI41_DMA_CH_CONFIG( PAL_CPPI41_DMA_RX_CH, PPFW_CPPI4x_HOST_TO_PP_PROXY_CHNUM(0)   ),
        PAL_CPPI41_DMA_CH_CONFIG( PAL_CPPI41_DMA_TX_CH, PPFW_CPPI4x_HOST_TO_PP_PROXY_CHNUM(0)   ),
        PAL_CPPI41_DMA_CH_CONFIG( PAL_CPPI41_DMA_RX_CH, PPFW_CPPI4x_HOST_TO_PP_PROXY_CHNUM(1)   ),
        PAL_CPPI41_DMA_CH_CONFIG( PAL_CPPI41_DMA_TX_CH, PPFW_CPPI4x_HOST_TO_PP_PROXY_CHNUM(1)   ),
    },
};



typedef struct channel_info_t
{
    Cppi4Queue  outputQueue;
    Cppi4Queue  freeQueue;
    int         bdnum;
    Ptr         bdPoolOffset;
    int         dmaChannel;
}
channel_info_t;


/************************************************************************/
/*      DOCSIS -> PP channels information                               */
/************************************************************************/
static  channel_info_t  rxIngress[CNI_CPPI4x_RX_DMA_CH_COUNT]=
{
    {
        .outputQueue.qMgr   = PPFW_CPPI4x_RX_INGRESS_QMGR,  
        .outputQueue.qNum   = PPFW_CPPI4x_RX_INGRESS_QNUM( PAL_CPPI4x_PRTY_LOW ),
        .freeQueue.qMgr     = CNI_CPPI4x_FD_QMGR,
        .freeQueue.qNum     = CNI_CPPI4x_FD_QNUM( PAL_CPPI4x_PRTY_LOW ),
        .bdnum              = CNI_RX_EMBEDDED_BD_NUM_LOW,
        .dmaChannel         = CNI_CPPI4x_RX_DMA_CHNUM(0)
    },
    {
        .outputQueue.qMgr   = PPFW_CPPI4x_RX_INGRESS_QMGR,  
        .outputQueue.qNum   = PPFW_CPPI4x_RX_INGRESS_QNUM( PAL_CPPI4x_PRTY_LOW ),
        .freeQueue.qMgr     = CNI_CPPI4x_FD_QMGR,
        .freeQueue.qNum     = CNI_CPPI4x_FD_QNUM( PAL_CPPI4x_PRTY_LOW ),
        .bdnum              = 0,
        .dmaChannel         = CNI_CPPI4x_RX_DMA_CHNUM(1)
    },
    {
        .outputQueue.qMgr   = PPFW_CPPI4x_RX_INGRESS_QMGR,  
        .outputQueue.qNum   = PPFW_CPPI4x_RX_INGRESS_QNUM( PAL_CPPI4x_PRTY_HIGH ),
        .freeQueue.qMgr     = CNI_CPPI4x_FD_QMGR,
        .freeQueue.qNum     = CNI_CPPI4x_FD_QNUM( PAL_CPPI4x_PRTY_HIGH ),
        .bdnum              = CNI_RX_EMBEDDED_BD_NUM_HIGH,
        .dmaChannel         = CNI_CPPI4x_RX_DMA_CHNUM(2)
    }
};
/************************************************************************/



typedef struct 
{
    Uint32  numDesc;
    Uint32  qNum;
    
    Ptr     firstDescPtr;
    Uint32  isAllocated;
    
    /* Desc config for the  group */
    Uint32 pktType;
} BDBlkInfo;

/* Embedded descriptor table:
 *  - This table aggregates the embedded descriptors in the system. It contains
 *  information about the first descriptor and allocation status of each
 *  division of the region. Generally this division is based on per driver or
 *  per Q basis.
 */
typedef struct 
{
    Ptr         buffDescRegionPtr;
    Uint32      qMgr;
    Uint32      numDesc;
    Uint32      szDesc;
    Uint32      numBlks;
    BDBlkInfo   BDBlk [PAL_CPPI41_FD_Q_LAST - PAL_CPPI41_FD_Q_BASE];
} EmbBDCfg;

EmbBDCfg ppEmbBDCfg_g = 
{
    .qMgr       = PPFW_CPPI4x_FD_QMGR,
    .numDesc    = CNI_RX_EMBEDDED_BD_NUM_LOW + CNI_RX_EMBEDDED_BD_NUM_HIGH, /* Must be same as in Cppi Cgf structure */
    .szDesc     = CNI_BD_SIZE,   /* Must be same as for specific region */
    .numBlks    = 2,            /* Change for each block added/removed */

    /* DOCSIS */
    .BDBlk[0].numDesc   = CNI_RX_EMBEDDED_BD_NUM_LOW,
    .BDBlk[0].qNum      = CNI_CPPI4x_FD_QNUM( PAL_CPPI4x_PRTY_LOW ),
    .BDBlk[0].pktType   = PAL_CPPI4_HOSTDESC_PKT_TYPE_ETH,

    .BDBlk[1].numDesc   = CNI_RX_EMBEDDED_BD_NUM_HIGH,
    .BDBlk[1].qNum      = CNI_CPPI4x_FD_QNUM( PAL_CPPI4x_PRTY_HIGH ),
    .BDBlk[1].pktType   = PAL_CPPI4_HOSTDESC_PKT_TYPE_ETH,
};

static int docsisMngInit (void);

static int secondaryPumaSrCppiInit(void)
{
    int i;
    PAL_Handle palHandle; 
    Cppi4Queue tmpQ;    /* Just used for filling Q info for opening */
    Cppi4BufPool tmpBufPool; /* Used for Init calls */

    printk(" Entered %s \n", __FUNCTION__);

    secondaryPumaCppiConfigTranslation( &cppi4InitCfgSecondarySR_g,     secondaryPumaGetDynamic1Addr()  );
    
    palHandle = PAL_cppi4Init ( &cppi4InitCfgSecondarySR_g , CPPI41_DOMAIN_SECONDARY_SR );
  
    if (!palHandle) 
    {
        printk(" %s: CPPI41 Library NOT initialized.\n", __FUNCTION__);
        return -1;
    }

    /************************************************************************/
    /*********** Init buffer pools used in the system ***********************/
    /************************************************************************/
    
    tmpBufPool.bMgr     = PAL_CPPI41_BUF_MGR_PARTITION_SR;

    /************************************************/
    /*                DOCSIS                        */
    /*                                              */
    tmpBufPool.bPool    = BMGR0_POOL00;
    if ((PAL_cppi4BufPoolInit(palHandle, tmpBufPool,
                        BMGR0_POOL00_REF_CNT,
                        BMGR0_POOL00_BUF_SIZE,
                        BMGR0_POOL00_BUF_COUNT )) == NULL)
    {
        printk ("PAL_cppi4BufPoolInit for pool %d FAILED.\n", tmpBufPool.bPool);
        return -1;
    }
    /*----------------------------------------------*/

    tmpBufPool.bPool    = BMGR0_POOL01;
    if ((PAL_cppi4BufPoolInit(palHandle, tmpBufPool,
                        BMGR0_POOL01_REF_CNT,
                        BMGR0_POOL01_BUF_SIZE,
                        BMGR0_POOL01_BUF_COUNT )) == NULL)
    {
        printk ("PAL_cppi4BufPoolInit for pool %d FAILED.\n", tmpBufPool.bPool);
        return -1;
    }
    /*----------------------------------------------*/
    
    tmpBufPool.bPool    = BMGR0_POOL04;
    if ((PAL_cppi4BufPoolInit(palHandle, tmpBufPool,
                        BMGR0_POOL04_REF_CNT,
                        BMGR0_POOL04_BUF_SIZE,
                        BMGR0_POOL04_BUF_COUNT )) == NULL)
    {
        printk ("PAL_cppi4BufPoolInit for pool %d FAILED.\n", tmpBufPool.bPool);
        return -1;
    }
    /*----------------------------------------------*/
    
    tmpBufPool.bPool    = BMGR0_POOL05;
    if ((PAL_cppi4BufPoolInit(palHandle, tmpBufPool,
                        BMGR0_POOL05_REF_CNT,
                        BMGR0_POOL05_BUF_SIZE,
                        BMGR0_POOL05_BUF_COUNT )) == NULL)
    {
        printk ("PAL_cppi4BufPoolInit for pool %d FAILED.\n", tmpBufPool.bPool);
        return -1;
    }
    /*----------------------------------------------*/
    
    tmpBufPool.bPool    = BMGR0_POOL12;
    if ((PAL_cppi4BufPoolInit(palHandle, tmpBufPool,
                        BMGR0_POOL12_REF_CNT,
                        BMGR0_POOL12_BUF_SIZE,
                        BMGR0_POOL12_BUF_COUNT )) == NULL)
    {
        printk ("PAL_cppi4BufPoolInit for pool %d FAILED.\n", tmpBufPool.bPool);
        return -1;
    }
    /************************************************/

    /************************************************/
    /*                MPEG I/F                      */
    /*                                              */
    tmpBufPool.bPool    = MPEG_CPPI4x_POOL_NUM(0);
    if ((PAL_cppi4BufPoolInit(palHandle, tmpBufPool,
                        BMGR0_POOL02_REF_CNT,
                        BMGR0_POOL02_BUF_SIZE,
                        BMGR0_POOL02_BUF_COUNT)) == NULL)
    {
        printk ("PAL_cppi4BufPoolInit for pool %d FAILED.\n", tmpBufPool.bPool);
        return -1;
    }
    /*----------------------------------------------*/

    tmpBufPool.bPool    = MPEG_ENCAP_CPPI4x_POOL_NUM(0);
    if ((PAL_cppi4BufPoolInit(palHandle, tmpBufPool,
                        BMGR0_POOL08_REF_CNT,
                        BMGR0_POOL08_BUF_SIZE,
                        BMGR0_POOL08_BUF_COUNT)) == NULL)
    {
        printk ("PAL_cppi4BufPoolInit for pool %d FAILED.\n", tmpBufPool.bPool);
        return -1;
    }
    /************************************************/

    /******************** Buffer pool Init done *****************************/



    /***************************************************************
     * Prepare CNI Rx channels
     * ====================
     */
    {
        int iCniChan;

        /* 
         * Prepare Cni Rx queues
         * ---------------------
         * Free queues are from Q[144+7].. 
         * Rx complete queues are from Q[56].. - Prefetcher queues
         */
        for (iCniChan=0; iCniChan<CNI_CPPI4x_RX_DMA_CH_COUNT; iCniChan++)
        {
        /* 
         * Open Docsis Cni Rx channels
         * -----------------------------
         * Free queues are from   Q[128+0]
         * Output queues are from Q[100+5]
         */
            {
                volatile Cppi4RxChInitCfg CniRxChInfo;
                PAL_Cppi4RxChHnd CniRxChHdl;

                CniRxChInfo.chNum               = rxIngress[iCniChan].dmaChannel;
                CniRxChInfo.dmaNum              = PAL_CPPI41_DMA_BLOCK1;
                CniRxChInfo.rxCompQueue.qMgr    = rxIngress[iCniChan].outputQueue.qMgr;
                CniRxChInfo.rxCompQueue.qNum    = rxIngress[iCniChan].outputQueue.qNum;
                CniRxChInfo.sopOffset = 0; // SOF skip=0

                CniRxChInfo.defDescType = CPPI41_DESC_TYPE_EMBEDDED;
                CniRxChInfo.u.embeddedPktCfg.fdQueue.qMgr = rxIngress[iCniChan].freeQueue.qMgr;
                CniRxChInfo.u.embeddedPktCfg.fdQueue.qNum = rxIngress[iCniChan].freeQueue.qNum;
                CniRxChInfo.u.embeddedPktCfg.numBufSlot = (EMSLOTCNT-1);
                CniRxChInfo.u.embeddedPktCfg.sopSlotNum = 1;

                CniRxChInfo.u.embeddedPktCfg.fBufPool[0].bMgr   = BUF_POOL_MGR0;  
                CniRxChInfo.u.embeddedPktCfg.fBufPool[1].bMgr   = BUF_POOL_MGR0;
                CniRxChInfo.u.embeddedPktCfg.fBufPool[2].bMgr   = BUF_POOL_MGR0;
                CniRxChInfo.u.embeddedPktCfg.fBufPool[3].bMgr   = BUF_POOL_MGR0;
                switch (iCniChan)
                {
                    case 0:
                        CniRxChInfo.u.embeddedPktCfg.fBufPool[0].bPool  = BMGR0_POOL00; 
                        CniRxChInfo.u.embeddedPktCfg.fBufPool[1].bPool  = BMGR0_POOL00; 
                        CniRxChInfo.u.embeddedPktCfg.fBufPool[2].bPool  = BMGR0_POOL04; 
                        CniRxChInfo.u.embeddedPktCfg.fBufPool[3].bPool  = BMGR0_POOL00; 
                	break;

                    case 1:
                        CniRxChInfo.u.embeddedPktCfg.fBufPool[0].bPool  = BMGR0_POOL01; 
                        CniRxChInfo.u.embeddedPktCfg.fBufPool[1].bPool  = BMGR0_POOL01; 
                        CniRxChInfo.u.embeddedPktCfg.fBufPool[2].bPool  = BMGR0_POOL04; 
                        CniRxChInfo.u.embeddedPktCfg.fBufPool[3].bPool  = BMGR0_POOL00; 
                    break;

                    case 2:
                        CniRxChInfo.u.embeddedPktCfg.fBufPool[0].bPool  = BMGR0_POOL05; 
                        CniRxChInfo.u.embeddedPktCfg.fBufPool[1].bPool  = BMGR0_POOL05; 
                        CniRxChInfo.u.embeddedPktCfg.fBufPool[2].bPool  = BMGR0_POOL05; 
                        CniRxChInfo.u.embeddedPktCfg.fBufPool[3].bPool  = BMGR0_POOL00; 
                    break;
                }

                printk(" Call PAL_cppi4RxChOpen channel=%d\n", CniRxChInfo.chNum);

                CniRxChHdl = PAL_cppi4RxChOpen (palHandle, (Cppi4RxChInitCfg *)(&CniRxChInfo), NULL);
                if(CniRxChHdl == NULL)
                {
                    printk(" Unable to open %d channel \n", CniRxChInfo.chNum);
                    return -1;
                }

                PAL_cppi4EnableRxChannel (CniRxChHdl, NULL);

                printk(" Call PAL_cppi4EnableRxChannel channel=%d Done\n", CniRxChInfo.chNum);
            }
        }
    }

    
    /************************************************************************/
    /*  Prepare Upstream related channels                                   */
    /************************************************************************/
    {
        volatile Cppi4RxChInitCfg  rxchInfo;
        PAL_Cppi4RxChHnd           rxchHdl;
        volatile Cppi4TxChInitCfg  txchInfo;
        PAL_Cppi4TxChHnd           txchHdl;
        Cppi4Queue                 fdQueue = {PAL_CPPI41_QUEUE_MGR_PARTITION_DOCSIS, CNI_CPPI4x_DOCSIS_US_FDQ};
        
        /* init co-proc TX channel */
        txchInfo.chNum          = CNI_CPPI4x_TX_CoP_DMA_CHNUM(0);
        txchInfo.dmaNum         = PAL_CPPI41_DMA_BLOCK1;
        txchInfo.tdQueue.qMgr   = DMA1_CPPI4x_FTD_QMGR;
        txchInfo.tdQueue.qNum   = DMA1_CPPI4x_FTD_QNUM;
        
        if (NULL == (txchHdl = PAL_cppi4TxChOpen( palHandle, (Cppi4TxChInitCfg *)(&txchInfo), NULL )))
        {
            printk(" Unable to open %d channel \n",txchInfo.chNum);
            return -1;
        }
        
        PAL_cppi4EnableTxChannel (txchHdl, NULL);
        
        /* init co-proc RX channel qman 1 queue */
        rxchInfo.chNum          = CNI_CPPI4x_RX_CoP_DMA_CHNUM(0);
        rxchInfo.dmaNum         = PAL_CPPI41_DMA_BLOCK1;
        rxchInfo.defDescType    = CPPI41_DESC_TYPE_EMBEDDED;
        rxchInfo.sopOffset=0;
        
        rxchInfo.u.embeddedPktCfg.fdQueue = fdQueue;
        rxchInfo.u.embeddedPktCfg.numBufSlot = 2;
        rxchInfo.u.embeddedPktCfg.sopSlotNum = 0;
        rxchInfo.u.embeddedPktCfg.fBufPool[0].bMgr  = BUF_POOL_MGR0;
        rxchInfo.u.embeddedPktCfg.fBufPool[0].bPool = BMGR0_POOL12;
        rxchInfo.u.embeddedPktCfg.fBufPool[1].bMgr  = BUF_POOL_MGR0;
        rxchInfo.u.embeddedPktCfg.fBufPool[1].bPool = BMGR0_POOL12;
        rxchInfo.u.embeddedPktCfg.fBufPool[2].bMgr  = BUF_POOL_MGR0;
        rxchInfo.u.embeddedPktCfg.fBufPool[2].bPool = BMGR0_POOL12;
        rxchInfo.u.embeddedPktCfg.fBufPool[3].bMgr  = BUF_POOL_MGR0;
        rxchInfo.u.embeddedPktCfg.fBufPool[3].bPool = BMGR0_POOL12;
        
        if (NULL == (rxchHdl = PAL_cppi4RxChOpen ( palHandle, (Cppi4RxChInitCfg *)(&rxchInfo), NULL )))
        {
            printk(" Unable to open %d channel \n",rxchInfo.chNum);
            return -1;
        }
        
        PAL_cppi4EnableRxChannel (rxchHdl, NULL);
    }

    printk(" %s : About to init BDs \n",__FUNCTION__);

    /************************************************************************/
    /*********** Setup Free Embedded descriptors    *************************/
    /************************************************************************/

    /************************************************/
    /*      Allocate region                         */
    /*                                              */
    ppEmbBDCfg_g.buffDescRegionPtr = 
        PAL_cppi4AllocDesc( palHandle,ppEmbBDCfg_g.qMgr, 
        ppEmbBDCfg_g.numDesc, 
        ppEmbBDCfg_g.szDesc );
    
    if (!ppEmbBDCfg_g.buffDescRegionPtr) 
    {
        printk (" %s : Embedded descriptor region allocation FAILED.\n",__FUNCTION__);
        return -1;
    }
    else
    {
        printk (" %s : Embedded descriptor region allocated [0x%08X] \n",__FUNCTION__, ppEmbBDCfg_g.buffDescRegionPtr );        
    }
    /************************************************/
    
    {
        Cppi4EmbdDesc* currBD;
        Cppi4EmbdDesc* currBDphysical;
        currBDphysical = (Cppi4EmbdDesc*)ppEmbBDCfg_g.buffDescRegionPtr;
        currBD = SECONDARY_TRANSLATE_CPPI(currBDphysical, secondaryPumaGetDynamic2Addr());
        
        for (i = 0; i < ppEmbBDCfg_g.numBlks; i++)
        {
            BDBlkInfo* BDBlk = &ppEmbBDCfg_g.BDBlk[i];
            int bd_cnt;
            PAL_Cppi4QueueHnd tmpQHnd; 
            
            tmpQ.qMgr = ppEmbBDCfg_g.qMgr;
            tmpQ.qNum = BDBlk->qNum;
            tmpQHnd = PAL_cppi4QueueOpen (palHandle, tmpQ);
            
            printk (" %s : Queue[%3d] has first BD at [0x%08X](Virt)\n",__FUNCTION__,tmpQ.qNum, currBD );

            for (bd_cnt = 0; bd_cnt < BDBlk->numDesc; bd_cnt++)
            {
                currBD->descInfo    = CPPI41_EM_DESCINFO_DTYPE_EMBEDDED |
                    CPPI41_EM_DESCINFO_SLOTCNT_MYCNT;
                currBD->tagInfo     = 0;
                currBD->pktInfo     = 
                    (BDBlk->pktType    << CPPI41_EM_PKTINFO_PKTTYPE_SHIFT) 
                    |(1                 << CPPI41_EM_PKTINFO_RETPOLICY_SHIFT)
                    |(1                 << CPPI41_EM_PKTINFO_PROTSPEC_SHIFT)
                    |(ppEmbBDCfg_g.qMgr << PAL_CPPI4_HOSTDESC_PKT_RETQMGR_SHIFT)
                    |(BDBlk->qNum       << PAL_CPPI4_HOSTDESC_PKT_RETQNUM_SHIFT);
                
                PAL_CPPI4_CACHE_WRITEBACK(currBD, ppEmbBDCfg_g.szDesc);
                
                PAL_cppi4QueuePush (tmpQHnd, 
                    (Ptr)currBDphysical, 
                    (ppEmbBDCfg_g.szDesc-24)/4, 
                    0/*!@@*/);
                
                currBD          = (Cppi4EmbdDesc*)((Uint32)currBD         + ppEmbBDCfg_g.szDesc);
                currBDphysical  = (Cppi4EmbdDesc*)((Uint32)currBDphysical + ppEmbBDCfg_g.szDesc);
            }
        }
    }
    /********************** Free Embedded desc setup Done  ******************/

    /* Initialize infa for recycling resources */
    secondaryPuma_setupRecycleInfra(palHandle);

    docsisMngInit();
    return 0;
}



static int docsisMngInit (void) 
{
    PAL_Handle          palHandle;
    Cppi4Queue          rxFreeQ;
    Cppi4Queue          rxQ;
    PAL_Cppi4QueueHnd   rxfdQueueHdl;
    int                 cnt;
    Cppi4HostDesc*      currBD;
    Cppi4HostDesc*      currBDphysical;
    Ptr                 currBuffer;
    Cppi4RxChInitCfg    rxCh;
    PAL_Cppi4RxChHnd    rxChHandle;
    /* get a handle to the PAL structure */
    if ((palHandle =  PAL_cppi4Init( NULL, CPPI41_DOMAIN_SECONDARY_SR )) == NULL) 
    {
        printk(" %s : PAL CPPI is not initialized yet \n", __FUNCTION__);
        return(-1);
    }   

    /*  open a queue objects one for free descriptor queue
     *  and one for rx queue 
     */ 

    rxQ.qMgr = CNI_MGMT_CPPI4x_RX_QMGR;
    rxQ.qNum = CNI_MGMT_CPPI4x_RX_QNUM(0);

    if (NULL == PAL_cppi4QueueOpen( palHandle, rxQ )) 
    {
        return -1;
    }

    rxFreeQ.qMgr = CNI_MGMT_CPPI4x_FBD_QMGR;
    rxFreeQ.qNum = CNI_MGMT_CPPI4x_FBD_QNUM(0);

    if (NULL == (rxfdQueueHdl = PAL_cppi4QueueOpen( palHandle, rxFreeQ ))) 
    {
        return -1;
    }


    /*
     * get a pointer to the BD region 
     * the region is already preallocated in the PAL init so just 
     * get a pointer to the bd area 
     */
    if ((currBDphysical = (Cppi4HostDesc *)PAL_cppi4AllocDesc( palHandle, CNI_MGMT_CPPI4x_FBD_QMGR, 
                                                               CMGR_CPPI4x_RX_HOST_BD_NUM,
                                                               CMGR_CPPI4x_RX_HOST_BD_SIZE )) == NULL)
    {
        printk( " %s : Failed to allocate Rx descriptor\n", __FUNCTION__);
        return -1;
    }

    currBD = SECONDARY_TRANSLATE_CPPI( currBDphysical, secondaryPumaGetDynamic2Addr() );
    
    /* allocate a memory for the buffers */
    if ( NULL == (currBuffer = secondaryPuma_memAllocSizeAligned( CMGR_CPPI4x_RX_HOST_BD_NUM * CMGR_CPPI4x_RX_HOST_BUFF_SIZE )) )
    {
        printk( " %s : Failed to allocate memory for RX buffer queue",__FUNCTION__);
        return -1;
    }
 
    /* need to consider the number of actually allocated buffers !!!!*/
    /* we don't really need 32 pair for management channel, do we? */
    for (cnt = 0; cnt < CMGR_CPPI4x_RX_HOST_BD_NUM; cnt++) 
    {
        /* Update the hardware descriptor */
        currBD->descInfo    = ((PAL_CPPI4_HOSTDESC_DESC_TYPE_HOST    << PAL_CPPI4_HOSTDESC_DESC_TYPE_SHIFT));
        currBD->tagInfo     = 0;
        currBD->pktInfo     = 
                              (PAL_CPPI4_HOSTDESC_PKT_TYPE_ETH       << PAL_CPPI4_HOSTDESC_PKT_TYPE_SHIFT)
                            | (PAL_CPPI4_HOSTDESC_PKT_RETPLCY_LINKED << PAL_CPPI4_HOSTDESC_PKT_RETPLCY_SHIFT)
                            | (PAL_CPPI4_HOSTDESC_DESC_LOC_OFFCHIP   << PAL_CPPI4_HOSTDESC_DESC_LOC_SHIFT)
                            | (rxFreeQ.qMgr                          << PAL_CPPI4_HOSTDESC_PKT_RETQMGR_SHIFT)
                            | (rxFreeQ.qNum                          << PAL_CPPI4_HOSTDESC_PKT_RETQNUM_SHIFT);

        currBD->buffLen     = 0;
        currBD->bufPtr      = 0;
        currBD->nextBDPtr   = 0;
        currBD->orgBuffLen  = CMGR_CPPI4x_RX_HOST_BUFF_SIZE;
        currBD->orgBufPtr   = PAL_CPPI4_VIRT_2_PHYS(currBuffer);

        PAL_CPPI4_CACHE_WRITEBACK( currBD, CMGR_CPPI4x_RX_HOST_BD_SIZE );

        PAL_cppi4QueuePush ( rxfdQueueHdl,
                             (Ptr) currBDphysical,
                             (CMGR_CPPI4x_RX_HOST_BD_SIZE - 24)/4,
                             0 );
    
        currBD          = (Cppi4HostDesc*) ((Uint32)currBD          + CMGR_CPPI4x_RX_HOST_BD_SIZE);
        currBDphysical  = (Cppi4HostDesc*) ((Uint32)currBDphysical  + CMGR_CPPI4x_RX_HOST_BD_SIZE);
        currBuffer =        (Ptr) ((Uint32)currBuffer + CMGR_CPPI4x_RX_HOST_BUFF_SIZE);
    }

    /* configure the DMA channel to use this queue*/

    rxCh.chNum =        CNI_CPPI4x_DOCSIS_TO_HOST_MGMT_CHNUM;
    rxCh.dmaNum =       PAL_CPPI41_DMA_BLOCK1;
    rxCh.defDescType =  CPPI41_DESC_TYPE_HOST;
    rxCh.sopOffset =    0;
    rxCh.rxCompQueue =  rxQ;
    rxCh.u.hostPktCfg.fdbQueue[0] = rxFreeQ;  
    rxCh.u.hostPktCfg.fdbQueue[1] = rxFreeQ;  
    rxCh.u.hostPktCfg.fdbQueue[2] = rxFreeQ;  
    rxCh.u.hostPktCfg.fdbQueue[3] = rxFreeQ;  

    
    if (NULL == (rxChHandle = PAL_cppi4RxChOpen (palHandle, &rxCh, NULL)))
    {
        printk( " %s : Failed to open Management DMA channel",__FUNCTION__);
        return(-1);
    }

    PAL_cppi4EnableRxChannel (rxChHandle, NULL);

    return 0;
}




















/************************************************************************/
/*                ____   ___   ____ ____ ___ ____   					*/
/*               |  _ \ / _ \ / ___/ ___|_ _/ ___|  					*/
/*               | | | | | | | |   \___ \| |\___ \  					*/
/*               | |_| | |_| | |___ ___) | | ___) | 					*/
/*               |____/ \___/ \____|____/___|____/  					*/
/*                                                                      */
/*                         ___       _ _                                */
/*                        |_ _|_ __ (_) |_                              */
/*                         | || '_ \| | __|                             */
/*                         | || | | | | |_                              */
/*                        |___|_| |_|_|\__|                             */
/*                                                                      */
/************************************************************************/
Cppi4InitCfg cppi4InitCfgSecondaryDOCSIS_g = 
{
    .resetLine              = 0,
    .appSpecificMemAllocFunc = secondaryPuma_memAllocSizeAligned,
        
    /************************************************************************/
    /*                                                                      */
    /************************************************************************/
    
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_DOCSIS].queueMgrRgnBase        = (Ptr) AVALANCHE_DOCSIS_SS_QMGR_RGN_BASE, 
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_DOCSIS].descMemRgnBase         = (Ptr) AVALANCHE_DOCSIS_SS_DESCMEM_RGN_BASE,     
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_DOCSIS].queueMgmtRgnBase       = (Ptr) AVALANCHE_DOCSIS_SS_QMGMT_RGN_BASE,
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_DOCSIS].queueStatusRgnBase     = (Ptr) AVALANCHE_DOCSIS_SS_QSTATUS_RGN_BASE,
    
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_DOCSIS].LinkingRAM0Base        = IO_VIRT2PHY(AVALANCHE_DOCSIS_SS_LINKING_RAM_BASE), 
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_DOCSIS].LinkingRAM0Size        = 1024, 
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_DOCSIS].LinkingRAM1Base        = IO_VIRT2PHY(AVALANCHE_DOCSIS_SS_LINKING_RAM_BASE+0x1000),
    
    /* Docsis MAC DS internal memory */
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_DOCSIS].descRegion[0].base     = (Ptr) IO_VIRT2PHY( AVALANCHE_DOCSIS_SS_DS_PACKET_RAM_BASE + 0x8000 ),
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_DOCSIS].descRegion[0].szDesc   = CNI_CPPI4x_DOCSIS_DS_DESC_SIZE, 
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_DOCSIS].descRegion[0].numDesc  = CNI_CPPI4x_DOCSIS_DS_DESC_NUM,
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_DOCSIS].descRegion[0].isOnChip = 1,    
    
    /* Docsis MAC US internal memory */
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_DOCSIS].descRegion[1].base     = (Ptr) IO_VIRT2PHY( AVALANCHE_DOCSIS_SS_US_PACKET_RAM_BASE ),
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_DOCSIS].descRegion[1].szDesc   = 128, 
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_DOCSIS].descRegion[1].numDesc  = 32,
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_DOCSIS].descRegion[1].isOnChip = 1,    
    
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_DOCSIS].descRegion[2].base     = 0,
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_DOCSIS].descRegion[2].szDesc   = CNI_CPPI4x_DOCSIS_US_DESC_SIZE, 
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_DOCSIS].descRegion[2].numDesc  = CNI_CPPI4x_DOCSIS_US_DESC_NUM,
    .queueMgrInfo[PAL_CPPI41_QUEUE_MGR_PARTITION_DOCSIS].descRegion[2].isOnChip = 0,
    
    /************************************************************************/
    /*                                                                      */
    /************************************************************************/
    
    .bufMgrBase[PAL_CPPI41_BUF_MGR_PARTITION_DOCSIS]    = (CSL_BufMgr_RegsOvly) AVALANCHE_DOCSIS_SS_BMGR_BASE,
    
    /************************************************************************/
    /*                                                                      */
    /************************************************************************/
    
    .dmaBlock[PAL_CPPI41_DMA_BLOCK0].globalCtrlBase     = (Ptr) AVALANCHE_DOCSIS_DMA0_GBLCFG_BASE,
    .dmaBlock[PAL_CPPI41_DMA_BLOCK0].chCtrlStatusBase   = (Ptr) AVALANCHE_DOCSIS_DMA0_CHNCFG_BASE,
    .dmaBlock[PAL_CPPI41_DMA_BLOCK0].schedCtrlBase      = (Ptr) AVALANCHE_DOCSIS_DMA0_SCHEDCFG_BASE,
    .dmaBlock[PAL_CPPI41_DMA_BLOCK0].schedTableBase     = (Ptr) AVALANCHE_DOCSIS_DMA0_SCHEDTBL_BASE,
    
    .dmaBlock[PAL_CPPI41_DMA_BLOCK0].tdFQueue.qMgr      = 0,
    .dmaBlock[PAL_CPPI41_DMA_BLOCK0].tdFQueue.qNum      = 0,
    
    .dmaBlock[PAL_CPPI41_DMA_BLOCK1].globalCtrlBase     = (Ptr) AVALANCHE_DOCSIS_DMA1_GBLCFG_BASE,
    .dmaBlock[PAL_CPPI41_DMA_BLOCK1].chCtrlStatusBase   = (Ptr) AVALANCHE_DOCSIS_DMA1_CHNCFG_BASE,
    .dmaBlock[PAL_CPPI41_DMA_BLOCK1].schedCtrlBase      = (Ptr) AVALANCHE_DOCSIS_DMA1_SCHEDCFG_BASE,
    .dmaBlock[PAL_CPPI41_DMA_BLOCK1].schedTableBase     = (Ptr) AVALANCHE_DOCSIS_DMA1_SCHEDTBL_BASE,
    
    .dmaBlock[PAL_CPPI41_DMA_BLOCK1].tdFQueue.qMgr      = PAL_CPPI41_QUEUE_MGR_PARTITION_DOCSIS,
    .dmaBlock[PAL_CPPI41_DMA_BLOCK1].tdFQueue.qNum      = 59,
    
    .dmaBlock[PAL_CPPI41_DMA_BLOCK0].schedTable.numEntries  = 1,
    .dmaBlock[PAL_CPPI41_DMA_BLOCK0].schedTable.entry = 
        { 
            PAL_CPPI41_DMA_CH_CONFIG( PAL_CPPI41_DMA_TX_CH, 0 ),
        },
    
    .dmaBlock[PAL_CPPI41_DMA_BLOCK1].schedTable.numEntries  = 8,
    .dmaBlock[PAL_CPPI41_DMA_BLOCK1].schedTable.entry = 
        {   
            PAL_CPPI41_DMA_CH_CONFIG( PAL_CPPI41_DMA_RX_CH, 0 ),
            PAL_CPPI41_DMA_CH_CONFIG( PAL_CPPI41_DMA_RX_CH, 1 ),
            PAL_CPPI41_DMA_CH_CONFIG( PAL_CPPI41_DMA_RX_CH, 2 ),
            PAL_CPPI41_DMA_CH_CONFIG( PAL_CPPI41_DMA_RX_CH, 3 ),
            PAL_CPPI41_DMA_CH_CONFIG( PAL_CPPI41_DMA_RX_CH, 4 ),
            PAL_CPPI41_DMA_CH_CONFIG( PAL_CPPI41_DMA_RX_CH, 5 ),
            PAL_CPPI41_DMA_CH_CONFIG( PAL_CPPI41_DMA_RX_CH, 6 ), 
            PAL_CPPI41_DMA_CH_CONFIG( PAL_CPPI41_DMA_RX_CH, 7 ),
        },
};


/************************************************************************/
/*                                                                      */
/*      DOCSIS Buffer Manager specific definition                       */
/*                                                                      */
/************************************************************************/
#define DS_EMBEDDED_DESC_RAM_BASE_ADDR_HOST         IO_PHY2VIRT((0x09040000 + 0x8000)) /* Host addressing */

typedef struct
{
    Cppi4EmbdDesc   hw;
    Uint32          psi[5];
}
docsisCppiDsDescType_t;


#define DS_EMBEDDED_DESC_BPI_CMD_TEMPLATE           0x21800000
#define DS_EMBEDDED_DESC_CRC_CMD_TEMPLATE           0x41800000
#define DS_EMBEDDED_DESC_PHS_CMD_TEMPLATE           0x11800000
#define DS_EMBEDDED_DESC_RX_FIFO_0_CMD_TEMPLATE     0x72000000
#define DS_EMBEDDED_DESC_RX_FIFO_1_CMD_TEMPLATE     0x080404C1


//************************************************************************
// Method:    secondaryPumaDocsisCppiInit
// FullName:  secondaryPumaDocsisCppiInit
// Access:    public static 
// Returns:   Int32
// Qualifier:
// Parameter: void
//************************************************************************
static Int32 secondaryPumaDocsisCppiInit(void)
{
    PAL_Handle          palHandle;  
    Cppi4BufPool        tmpBufPool; /* Used for Init calls */
    int channel;

    printk(" Entered %s \n", __FUNCTION__);

    secondaryPumaCppiConfigTranslation( &cppi4InitCfgSecondaryDOCSIS_g, secondaryPumaGetDocsisAddr() );

    palHandle = PAL_cppi4Init( &cppi4InitCfgSecondaryDOCSIS_g , CPPI41_DOMAIN_SECONDARY_DOCSIS );

    if (!palHandle) 
    {
        printk(" %s: CPPI41 Library NOT initialized.\n", __FUNCTION__);
        return -1;
    }
    
    /************************************************************************/
    /*  Buffer Pool Initialization                                          */
    /************************************************************************/
    tmpBufPool.bMgr     = PAL_CPPI41_BUF_MGR_PARTITION_DOCSIS;
    tmpBufPool.bPool    = BMGR1_POOL00;

    if ((PAL_cppi4BufPoolDirectInit(palHandle, tmpBufPool,
        BMGR1_POOL00_REF_CNT,
        BMGR1_POOL00_BUF_SIZE, 
        BMGR1_POOL00_BUF_COUNT,
        (Ptr)IO_VIRT2PHY( AVALANCHE_DOCSIS_SS_DS_PACKET_RAM_BASE ) )) == NULL)
    {
        printk ("PAL_cppi4BufPoolDirectInit for pool %d FAILED.\n", tmpBufPool.bPool);
        return -1;
    }
    /************************************************************************/


    /************************************************************************/
    /*  Format descriptors and fill in the free queue for DOCSIS Downstream */
    /************************************************************************/
    {
        docsisCppiDsDescType_t*     currBD;
        docsisCppiDsDescType_t*     currBDphysical;
        int bd_cnt;
        PAL_Cppi4QueueHnd tmpQHnd; 
        Cppi4Queue tmpQ;
        
        currBD =         (docsisCppiDsDescType_t*) SECONDARY_TRANSLATE_CPPI( DS_EMBEDDED_DESC_RAM_BASE_ADDR_HOST, secondaryPumaGetDocsisAddr() );
        currBDphysical = (docsisCppiDsDescType_t*) IO_VIRT2PHY( AVALANCHE_DOCSIS_SS_DS_PACKET_RAM_BASE + 0x8000 );
        
        tmpQ.qMgr = PAL_CPPI41_QUEUE_MGR_PARTITION_DOCSIS;
        tmpQ.qNum = CNI_CPPI4x_DOCSIS_DS_FDQ;
        tmpQHnd = PAL_cppi4QueueOpen ( palHandle, tmpQ );

        memset((void *)currBD, 0, CNI_CPPI4x_DOCSIS_DS_DESC_SIZE*CNI_CPPI4x_DOCSIS_DS_DESC_NUM);
        
        for (bd_cnt = 0; bd_cnt < CNI_CPPI4x_DOCSIS_DS_DESC_NUM; bd_cnt++)
        {
            currBD->hw.descInfo    = 
                  CPPI41_EM_DESCINFO_DTYPE_EMBEDDED 
                /*| CPPI41_EM_DESCINFO_SLOTCNT_MYCNT */
                | (5 << CPPI41_EM_DESCINFO_PSWSIZE_SHIFT);
            currBD->hw.tagInfo     = 0;
            currBD->hw.pktInfo     = 
                 (1                << CPPI41_EM_PKTINFO_RETPOLICY_SHIFT)
                |(1                << CPPI41_EM_PKTINFO_ONCHIP_SHIFT)
                |(tmpQ.qMgr        << PAL_CPPI4_HOSTDESC_PKT_RETQMGR_SHIFT)
                |(tmpQ.qNum        << PAL_CPPI4_HOSTDESC_PKT_RETQNUM_SHIFT);

            currBD->psi[0]  = DS_EMBEDDED_DESC_BPI_CMD_TEMPLATE;
            currBD->psi[1]  = DS_EMBEDDED_DESC_CRC_CMD_TEMPLATE;
            currBD->psi[2]  = DS_EMBEDDED_DESC_PHS_CMD_TEMPLATE;
            currBD->psi[3]  = DS_EMBEDDED_DESC_RX_FIFO_0_CMD_TEMPLATE;
            currBD->psi[4]  = DS_EMBEDDED_DESC_RX_FIFO_1_CMD_TEMPLATE;

            PAL_CPPI4_CACHE_WRITEBACK(currBD, CNI_CPPI4x_DOCSIS_DS_DESC_SIZE);
            
            PAL_cppi4QueuePush (tmpQHnd, (Ptr)currBDphysical, 
                (CNI_CPPI4x_DOCSIS_DS_DESC_SIZE-24)/4, 0);
            
            currBD          = (docsisCppiDsDescType_t*)((Uint32)currBD          + CNI_CPPI4x_DOCSIS_DS_DESC_SIZE);
            currBDphysical  = (docsisCppiDsDescType_t*)((Uint32)currBDphysical  + CNI_CPPI4x_DOCSIS_DS_DESC_SIZE);
        }
    }
    /************************************************************************/
    

    /************************************************************************/
    /*  DMA Rx Channels initialization.                                     */
    /************************************************************************/
    for (channel=0; channel<8; channel++)
    {
        Cppi4RxChInitCfg rxCh;
        PAL_Cppi4RxChHnd cppi4RxChHnd;

        /* Set up Rx channel */
        rxCh.chNum              = channel; 
        rxCh.dmaNum             = PAL_CPPI41_DMA_BLOCK1;
        rxCh.defDescType        = CPPI41_DESC_TYPE_EMBEDDED;
        rxCh.sopOffset          = 0;
        rxCh.rxCompQueue.qMgr   = PAL_CPPI41_QUEUE_MGR_PARTITION_DOCSIS;
        rxCh.rxCompQueue.qNum   = 0;
        rxCh.u.embeddedPktCfg.fdQueue.qMgr = PAL_CPPI41_QUEUE_MGR_PARTITION_DOCSIS;
        rxCh.u.embeddedPktCfg.fdQueue.qNum = CNI_CPPI4x_DOCSIS_DS_FDQ;
        rxCh.u.embeddedPktCfg.numBufSlot = (EMSLOTCNT-1);
        rxCh.u.embeddedPktCfg.sopSlotNum = 0;
        rxCh.u.embeddedPktCfg.fBufPool[0].bMgr  = PAL_CPPI41_BUF_MGR_PARTITION_DOCSIS;
        rxCh.u.embeddedPktCfg.fBufPool[0].bPool = BMGR1_POOL00;
        rxCh.u.embeddedPktCfg.fBufPool[1].bMgr  = PAL_CPPI41_BUF_MGR_PARTITION_DOCSIS;
        rxCh.u.embeddedPktCfg.fBufPool[1].bPool = BMGR1_POOL00;
        rxCh.u.embeddedPktCfg.fBufPool[2].bMgr  = PAL_CPPI41_BUF_MGR_PARTITION_DOCSIS;
        rxCh.u.embeddedPktCfg.fBufPool[2].bPool = BMGR1_POOL00;
        rxCh.u.embeddedPktCfg.fBufPool[3].bMgr  = PAL_CPPI41_BUF_MGR_PARTITION_DOCSIS;
        rxCh.u.embeddedPktCfg.fBufPool[3].bPool = BMGR1_POOL00;
        cppi4RxChHnd        = PAL_cppi4RxChOpen( palHandle, &rxCh, NULL );

        /* Enable Rx channel */
        PAL_cppi4EnableRxChannel (cppi4RxChHnd, NULL);
    }
    /************************************************************************/


    /************************************************************************/
    /*  DMA Tx Channel initialization.                                      */
    /************************************************************************/
    {
        Cppi4TxChInitCfg txCh;
        PAL_Cppi4TxChHnd cppi4TxChHnd;

        /* Set up Tx channel */
        txCh.chNum          = 0;
        txCh.dmaNum         = PAL_CPPI41_DMA_BLOCK0;
        txCh.tdQueue.qMgr   = PAL_CPPI41_BUF_MGR_PARTITION_DOCSIS;
        txCh.tdQueue.qNum   = CNI_CPPI4x_DOCSIS_DS_CoP_Q;

        cppi4TxChHnd        = PAL_cppi4TxChOpen( palHandle, &txCh, NULL );

        /* Enable Rx channel */
        PAL_cppi4EnableTxChannel (cppi4TxChHnd, NULL);
    }
    /************************************************************************/

    return (0);
}



/*
#define SECONDARY_TRANSLATE_CPPI(addr, secBase)      (IO_VIRT2PHY(addr) - PUMA5_CPPI_BASE_PHYS + secBase)
#define SECONDARY_TRANSLATE_DOCSIS(addr, secBase)    (IO_VIRT2PHY(addr) - PUMA5_CPPI_BASE_PHYS + secBase)
*/




/**************************************************************************/
/*! \fn void secondaryPumaCppiInit(void)
 **************************************************************************
 *  \brief Initialize the secondary-puma's CPPI domains.
 *  \return none.
 **************************************************************************/
int secondaryPumaCppiInit(void)
{
//    secondaryPumaCppiPsc();
//    secondaryPumaSrCppiInit();
    secondaryPumaDocsisCppiInit();

   return 0;
}

