/*
 *
 * secondary_puma_driver.h
 * Description:
 * declaration of functions and types used to control the Secondary Puma chip.
 *
 * Copyright (C) 2008 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifndef _HAL_SECONDARY_PUMA_DRIVER_H_
#define _HAL_SECONDARY_PUMA_DRIVER_H_

#include <asm/ioctl.h>
#include "sys_ptypes.h"
#include "soc_modules.h"
#include "soc_interface_driver.h"

/**************************************************************************/
/*      INTERFACE  Defines and Structs                                    */
/**************************************************************************/
#define SECONDARY_PUMA_DRIVER_DEV_NAME  SOC_INTERFACE_DRIVER_DEV_NAME

/**********************************************/
/* Definitions for Secondary Puma Driver      */
/**********************************************/

/* IOCTL commands:

   If you are adding new ioctl's to the kernel, you should use the _IO
   macros defined in <linux/ioctl.h> _IO macros are used to create ioctl numbers:

	_IO(type, nr)         - an ioctl with no parameter. 
   _IOW(type, nr, size)  - an ioctl with write parameters (copy_from_user), kernel would actually read data from user space 
   _IOR(type, nr, size)  - an ioctl with read parameters (copy_to_user), kernel would actually write data to user space 
   _IOWR(type, nr, size) - an ioctl with both write and read parameters 

   'Write' and 'read' are from the user's point of view, just like the
	system calls 'write' and 'read'.  For example, a SET_FOO ioctl would
	be _IOW, although the kernel would actually read data from user space;
	a GET_FOO ioctl would be _IOR, although the kernel would actually write
	data to user space.

	The first argument to _IO, _IOW, _IOR, or _IOWR is an identifying letter
	or number from the SoC_ModuleIds_e enum located in this file. 

	The second argument to _IO, _IOW, _IOR, or _IOWR is a sequence number
	to distinguish ioctls from each other.  

   The third argument to _IOW, _IOR, or _IOWR is the type of the data going
   into the kernel or coming out of the kernel (e.g.  'int' or 'struct foo').

   NOTE!  Do NOT use sizeof(arg) as the third argument as this results in 
   your ioctl thinking it passes an argument of type size_t.

*/
#define	SECONDARY_PUMA_WRITE_PHY_REG            _IOW (SOC_SECONDARY_PUMA_MODULE_ID, 1, RemoteRegInfo_t)
#define	SECONDARY_PUMA_READ_PHY_REG             _IOWR(SOC_SECONDARY_PUMA_MODULE_ID, 2, RemoteRegInfo_t)
#define	SECONDARY_PUMA_WRITE_PHY_REG_IMMEDIATE  _IOW (SOC_SECONDARY_PUMA_MODULE_ID, 3, RemoteRegInfo_t)
#define	SECONDARY_PUMA_READ_PHY_REG_IMMEDIATE   _IOWR(SOC_SECONDARY_PUMA_MODULE_ID, 4, RemoteRegInfo_t)
#define	SECONDARY_PUMA_WRITE_MAC_REG            _IOW (SOC_SECONDARY_PUMA_MODULE_ID, 5, RemoteMacRegInfo_t)
#define	SECONDARY_PUMA_READ_MAC_REG             _IOWR(SOC_SECONDARY_PUMA_MODULE_ID, 6, RemoteMacRegInfo_t)
#define	SECONDARY_PUMA_WRITE_MAC_BUFF           _IOW (SOC_SECONDARY_PUMA_MODULE_ID, 7, RemoteMacCopyInfo_t)
#define	SECONDARY_PUMA_READ_MAC_BUFF            _IOWR(SOC_SECONDARY_PUMA_MODULE_ID, 8, RemoteMacCopyInfo_t)
#define	SECONDARY_PUMA_DRIVER_INIT              _IOW (SOC_SECONDARY_PUMA_MODULE_ID, 9, Uint32)
#define	SECONDARY_PUMA_GET_SOC_ADDR             _IOR (SOC_SECONDARY_PUMA_MODULE_ID, 10, Uint32)
#define	SECONDARY_PUMA_EN_RF_SPLITTER           _IOW (SOC_SECONDARY_PUMA_MODULE_ID, 11, Uint32)
#define	SECONDARY_PUMA_MPEG_IN_ENABLE           _IO  (SOC_SECONDARY_PUMA_MODULE_ID, 12)
#define	SECONDARY_PUMA_BUFFER_CONTROL           _IO  (SOC_SECONDARY_PUMA_MODULE_ID, 13)
#define	SECONDARY_PUMA_PROXY_LOADED             _IO  (SOC_SECONDARY_PUMA_MODULE_ID, 14)

/* SECONDARY_PUMA_IOCTL_MAXNR: In case of adding new Secondary-Puma IOCTL commands this number must be update */
#define SECONDARY_PUMA_IOCTL_MAXNR 14

/**************************************************************************/
/*! \fn void secondaryPumaRegisterSoCModule (SoCDriverOperations_t **mpeg_out_operations)
 **************************************************************************
 *  \brief Register the secondary-Puma driver function pointers object.
 *  \param[in] **secondary_puma_operations_out - pointer to secondary-Puma operations.
 *  \return none.
 **************************************************************************/
#ifdef __KERNEL__
void secondaryPumaRegisterSoCModule (SoCDriverOperations_t **secondary_puma_operations_out);

/**************************************************************************/
/*! \fn Uint32 secondaryPumaGetSoCAddr (void)
 **************************************************************************
 *  \brief Get the start address of the local VLYNQ window that represents 
 *  \brief the remote (Secondary-Puma) SoC Addr.
 *  \return Local VLYNQ window that represents the remote SoC Addr.
 **************************************************************************/
Uint32 secondaryPumaGetSoCAddr (void);

/**************************************************************************/
/*! \fn int secondaryPumaGetDynamic1Addr ()
 **************************************************************************
 *  \brief Get the start address of the local VLYNQ window that represents 
 *  \brief the first dynamic memory mapping on the secondary puma.
 *  \param[out] *soc_addr - Local VLYNQ window.
 *  \return 0 on success of -1 on fail.
 **************************************************************************/
Uint32 secondaryPumaGetDynamic1Addr (void);

/**************************************************************************/
/*! \fn int secondaryPumaGetDynamic2Addr ()
 **************************************************************************
 *  \brief Get the start address of the local VLYNQ window that represents 
 *  \brief the second dynamic memory mapping on the secondary puma.
 *  \param[out] *soc_addr - Local VLYNQ window.
 *  \return 0 on success of -1 on fail.
 **************************************************************************/
Uint32 secondaryPumaGetDynamic2Addr (void);

/**************************************************************************/
/*! \fn Uint32 secondaryPumaGetDocsisAddr (void)
 **************************************************************************
 *  \brief Get the start address of the local VLYNQ window that represents 
 *  \brief the remote (Secondary-Puma) DOCSIS Addr.
 *  \return Local VLYNQ window that represents the remote DOCSIS Addr.
 **************************************************************************/
Uint32 secondaryPumaGetDocsisAddr (void);

/**************************************************************************/
/*! \fn int secondaryPumaIsInitialized ()
 **************************************************************************
 *  \brief Gets the secondaryPuma_driver_init variable. 
 **************************************************************************/
Uint32 secondaryPumaIsInitialized (void);

void secondaryPumaWriteDocsisMacReg(unsigned int regOffset, unsigned int value);
void secondaryPumaReadDocsisMacReg(unsigned int regOffset, unsigned int *value);
void secondaryPumaGetReadyToTransfer(unsigned int *lockKey);
void secondaryPumaFinishTransfer(unsigned int lockKey);

#endif /* __KERNEL__ */

#endif  /* _HAL_MPEG_SOC_DRIVER_H_ */
