/*
 *
 * hal_vlynq.h
 * Description:
 * HAL vlynq interface header file
 *
 * Copyright (C) 2008 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifndef _HAL_VLYNQ_H_
#define _HAL_VLYNQ_H_

/**************************************************************************/
/*      INCLUDES                                                          */
/**************************************************************************/
#include <_tistdtypes.h>

/**************************************************************************/
/*      INTERFACE  Defines and Structs                                    */
/**************************************************************************/
typedef enum vlynqType_e
{
    VLYNQ_LOCAL = 0x00,		/* Local VLYNQ registers */
    VLYNQ_REMOTE = 0x80,	/* Remote VLYNQ registers */

} vlynqType_e;

typedef enum vlynqClock_e
{
    VLYNQ_SOURCE_CLOCK,		/* Clock direction: Output */
    VLYNQ_SINK_CLOCK,		/* Clock direction: input */

} vlynqClock_e;

/**************************************************************************/
/*      INTERFACE  Functions                                              */
/**************************************************************************/

/* Reset VLYNQ link: reset = 1 reset, reset = 0 out of reset */
int VLYNQ_reset_link( unsigned int set );

/* Get VLYNQ link status: 1 Link up, 0 link down */
int VLYNQ_get_link_status( void );

/* poll until VLYNQ link is "UP", or max retry exceeded */
int VLYNQ_poll_for_link_up( unsigned int maxRetry );

/* Setup clock divider: 0 to 7 (+1) */
int VLYNQ_setup_clock_divider( unsigned int div, vlynqType_e type );

/* Setup clock direction: Source (output) or Sink (input) */
int VLYNQ_setup_clock_direction( vlynqClock_e clock, vlynqType_e type );

/* Setup interrupt configuration */
int VLYNQ_setup_interrupt_configuration( Bool intlocal, Bool int2cfg, vlynqType_e type );

/* Enable the remote PHY interrupt */
int VLYNQ_enable_remote_phy_interrupt( void );

/* Enable the remote MAC interrupt */
int VLYNQ_enable_remote_mac_interrupt( void );

int VLYNQ_secondary_puma_set_mem_access_start(void);
int VLYNQ_secondary_puma_can_mem_access_start(void);
int VLYNQ_secondary_puma_set_mem_access_finish(void);
int VLYNQ_secondary_puma_is_mem_access_finish(void);

/* Clear PHY interrupt */
int VLYNQ_clear_phy_interrupt( void );

/* Clear MAC interrupt */
int VLYNQ_clear_mac_interrupt( void );

/* determine if a phy interrupt (bit6) is pending */
int VLYNQ_is_phy_interrupt_pending( void );

/* determine if a mac interrupt (bit2) is pending */
int VLYNQ_is_mac_interrupt_pending( void );

/* TX region: The address which will be used for VLYNQ access */
int VLYNQ_setup_tx_region( unsigned int address, vlynqType_e type );

/* RX regions: The address windows which will be available to VLYNQ */
int VLYNQ_setup_rx_region( unsigned int address, unsigned int size, unsigned int index, vlynqType_e type );

/* Reset slave device using GPIO */
int VLYNQ_reset_slave( void );

/* Reset secondary puma using GPIO */
int VLYNQ_reset_secondary( void );     

/* Setup RTM */
int VLYNQ_setup_rtm(vlynqType_e type, unsigned int skew);


#endif
